SystemOrganization addCategory: #'Xpf-CodeTalk'!
SystemOrganization addCategory: #'Xpf-CodeTalk-Commands'!
SystemOrganization addCategory: #'Xpf-CodeTalk-Core'!
SystemOrganization addCategory: #'Xpf-CodeTalk-Tests'!
SystemOrganization addCategory: #'Xpf-CodeTalk-Morphs'!
SystemOrganization addCategory: #'Xpf-CodeTalk-Models'!
SystemOrganization addCategory: #'Xpf-CodeTalk-Versions'!

!DataStream methodsFor: '*xpf-codetalk' stamp: 'mt 3/24/2009 15:43'!
references

    ^ Dictionary new! !

TestCase subclass: #XpfCtComplexAttributeTest
	instanceVariableNames: 'sut'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Tests'!

!XpfCtComplexAttributeTest methodsFor: 'running' stamp: 'mt 5/5/2009 15:04'!
setUp

	super setUp.
	sut := XpfComplexAttribute forTag: #bad.
	sut addChatLine: 'She sells sea shells at the sea shore.'.! !

!XpfCtComplexAttributeTest methodsFor: 'tests' stamp: 'mt 5/5/2009 15:16'!
testInstanceCreation

	self
		should: sut tag = #bad;
		should: sut creator = Utilities authorInitials;
		should: sut chat size = 1;
		should: (sut chat first at: #author) = Utilities authorInitials;
		should: sut timestamp <= (sut chat first at: #timestamp)! !

!XpfCtComplexAttributeTest methodsFor: 'tests' stamp: 'mt 5/2/2009 15:35'!
testScanning

	| stream attr |
	stream := (String new: 500) writeStream.
	sut writeScanOn: stream.
	attr := XpfComplexAttribute scanFrom:
		(stream contents readStream position: 1). "Skip first 'h' to identify complex attribute. See RunArray class>>scanFrom:"
	
	self
		should: sut tag = attr tag;
		should: sut timestamp = attr timestamp;
		should: sut creator = attr creator;
		should: sut chat size = attr chat size;
		should: (sut chat first at: #author) = (attr chat first at: #author);
		should: (sut chat first at: #timestamp) = (attr chat first at: #timestamp);
		should: (sut chat first at: #message) = (attr chat first at: #message)! !

!XpfCtComplexAttributeTest methodsFor: 'tests' stamp: 'mt 5/18/2009 10:31'!
testTagAcccessing

	self should: sut tag = #bad.
	sut tag: #'0'.
	self should: sut tag isNil.
	sut tag: #talk.
	self should: sut tag = #talk.
	sut tag: #shouldNotBeAllowed.
	self should: sut tag isNil.! !

TestCase subclass: #XpfCtMonticelloTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Tests'!

!XpfCtMonticelloTest class methodsFor: 'marking' stamp: 'cs 5/4/2009 02:05'!
markCollection: anAttribute

	| source ref |
	ref := MethodReference class: Collection selector: #do:.
	source := ref sourceCode.
	(source attributesOfKind: XpfComplexAttribute)
		do: [:att | source removeAttribute: att ].
	source addAttribute: anAttribute from: 1 to: 1.
	Collection compile: source classified: ref category.
	^ ref! !

!XpfCtMonticelloTest class methodsFor: 'marking' stamp: 'mt 5/5/2009 15:05'!
markCollectionDoBad
	^ self markCollection: (XpfComplexAttribute forTag: #bad)! !

!XpfCtMonticelloTest class methodsFor: 'marking' stamp: 'cs 5/2/2009 12:58'!
markCollectionDoNormal

	| source ref |
	ref := MethodReference class: Collection selector: #do:.
	source := ref sourceCode.
	Collection compile: source asString classified: ref category.
	^ ref! !

!XpfCtMonticelloTest methodsFor: 'tests' stamp: 'mt 5/5/2009 15:05'!
testAddCodeMarkup
	| ref source md snapshotBin md2 color |
	ref := self class markCollectionDoBad.
	source := ref sourceCode.
	color := ((source attributesAt: 1) detect: [ :attr | attr isKindOf: XpfComplexAttribute ]) color.
	md := (Collection compiledMethodAt: #do:) methodReference asMethodDefinition.
	snapshotBin := MCMczWriter new serializeInBinary: (MCSnapshot fromDefinitions: {  md  }).
	md2 := (DataStream on: ((RWBinaryOrTextStream with: snapshotBin)
			reset;
			yourself)) next definitions first.
	self
		should: md2 source isString;
		should: md2 sourceText isText;
		should: md source = md2 source;
		should: ((md2 sourceText attributesAt: 1) anySatisfy: [ :attr | (attr isKindOf: XpfComplexAttribute) and: [ attr color = color ] ]).

	"Clean up"
	self class markCollectionDoNormal! !

!XpfCtMonticelloTest methodsFor: 'tests' stamp: 'mt 4/20/2009 16:17'!
testDefinitionFromReference

	| ref cm md |
	ref := MethodReference class: Collection selector: #do:.
	cm := Collection compiledMethodAt: #do:.
	md := ref asMethodDefinition.
	
	self
		should: cm getSource = ref sourceCode;
		should: ref sourceCode isText;
		should: ref source isString;
		should: md source isString;
		should: md sourceText isText.! !

TestCase subclass: #XpfCtTagSearchTest
	instanceVariableNames: 'sut badText att'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Tests'!

!XpfCtTagSearchTest methodsFor: 'helping' stamp: 'mt 5/5/2009 15:08'!
searchCollection: aBlock

	| helper |
	helper := XpfCtTagSearchHelper new.
	helper matchAttributeBad.
	aBlock value: helper.
	helper searchClass: Collection.
	^ helper results! !

!XpfCtTagSearchTest methodsFor: 'running' stamp: 'mt 5/5/2009 15:06'!
setUp

	super setUp.
	badText := 'int main(int argc, char** args) {printf("%s\n", args[0]);}' asText.
	att := XpfComplexAttribute forTag: #bad.
	badText 
		addAttribute: att
		from: 10
		to: 15.
	sut := XpfCtTagSearchHelper new.
	XpfCtMonticelloTest markCollectionDoBad.! !

!XpfCtTagSearchTest methodsFor: 'running' stamp: 'cs 5/2/2009 16:39'!
tearDown

	super tearDown.
	XpfCtMonticelloTest markCollectionDoNormal.! !

!XpfCtTagSearchTest methodsFor: 'testing' stamp: 'cs 5/2/2009 16:37'!
testBadCode

	sut matchAttributeBad.
	self should: (sut searchText: badText).! !

!XpfCtTagSearchTest methodsFor: 'testing' stamp: 'mt 5/5/2009 15:08'!
testCreator

	XpfCtMonticelloTest markCollection: (XpfComplexAttribute forTag: #bad).
	self
		should:   (self searchCollection: [:helper |
			helper matchCreator: Utilities authorInitials, 'X']) isEmpty;
		shouldnt: (self searchCollection: [:helper |
			helper matchCreator: Utilities authorInitials]) isEmpty.! !

!XpfCtTagSearchTest methodsFor: 'testing' stamp: 'mt 5/5/2009 15:08'!
testKeyword

	XpfCtMonticelloTest markCollection:
		((XpfComplexAttribute forTag: #bad) addChatLine: 'this is a test').
	self
		should:   (self searchCollection: [:helper | helper matchKeyword: 'tis']) isEmpty;
		shouldnt: (self searchCollection: [:helper | helper matchKeyword: 'tes']) isEmpty.! !

!XpfCtTagSearchTest methodsFor: 'testing' stamp: 'mt 5/5/2009 15:08'!
testSearchMethodCategory

	sut matchAttributeBad.
	self should: sut results isEmpty.
	sut searchCategory: 'Collections'.
	self
		shouldnt: sut results isEmpty;
		should: (sut results anySatisfy: [:ref |
			ref classIsMeta not &
			(ref classSymbol = #Collection) &
			(ref methodSymbol = #do:)]).! !

!XpfCtTagSearchTest methodsFor: 'testing' stamp: 'mt 5/5/2009 15:08'!
testSearchMethodClass

	sut matchAttributeBad.
	self should: sut results isEmpty.
	sut searchClass: Collection.
	self
		should: sut results size = 1;
		shouldnt: sut results first classIsMeta;
		should: sut results first classSymbol = #Collection;
		should: sut results first methodSymbol = #do:.! !

!XpfCtTagSearchTest methodsFor: 'testing' stamp: 'mt 5/5/2009 21:50'!
testSearchMethodGlobal

	sut matchAttributeBad.
	self should: sut results isEmpty.
	
	sut searchGlobal.
	
	self
		shouldnt: sut results isEmpty;
		should: (sut results anySatisfy: [:ref |
			ref classIsMeta not &
			(ref classSymbol = #Collection) &
			(ref methodSymbol = #do:)]).! !

!XpfCtTagSearchTest methodsFor: 'testing' stamp: 'mt 5/5/2009 15:09'!
testTimeLastMonth

	XpfCtMonticelloTest markCollection:
		((XpfComplexAttribute forTag: #bad) timestamp:
			TimeStamp now asDate onPreviousMonth asTimeStamp).
	self
		should:   (self searchCollection: [:helper | helper matchTime: #today]) isEmpty;
		should:   (self searchCollection: [:helper | helper matchTime: #yesterday]) isEmpty;
		should:   (self searchCollection: [:helper | helper matchTime: #thisWeek]) isEmpty;
		should:   (self searchCollection: [:helper | helper matchTime: #lastWeek]) isEmpty;
		should:   (self searchCollection: [:helper | helper matchTime: #thisMonth]) isEmpty;
		shouldnt: (self searchCollection: [:helper | helper matchTime: #lastMonth]) isEmpty.! !

!XpfCtTagSearchTest methodsFor: 'testing' stamp: 'mt 5/5/2009 15:09'!
testTimeToday

	XpfCtMonticelloTest markCollection:
		((XpfComplexAttribute forTag: #bad) timestamp: TimeStamp now).
	self
		shouldnt: (self searchCollection: [:helper | helper matchTime: #today]) isEmpty;
		should:   (self searchCollection: [:helper | helper matchTime: #yesterday]) isEmpty;
		shouldnt: (self searchCollection: [:helper | helper matchTime: #thisWeek]) isEmpty;
		should:   (self searchCollection: [:helper | helper matchTime: #lastWeek]) isEmpty;
		shouldnt: (self searchCollection: [:helper | helper matchTime: #thisMonth]) isEmpty;
		should:   (self searchCollection: [:helper | helper matchTime: #lastMonth]) isEmpty.! !

!XpfCtTagSearchTest methodsFor: 'testing' stamp: 'mt 5/5/2009 15:09'!
testTimeYesterday

	XpfCtMonticelloTest markCollection:
		((XpfComplexAttribute forTag: #bad) timestamp: TimeStamp now - (Duration days: 1)).
	self should:   (self searchCollection: [:helper | helper matchTime: #today]) isEmpty.
	self shouldnt: (self searchCollection: [:helper | helper matchTime: #yesterday]) isEmpty.
	TimeStamp now dayOfWeekName = Week startDay ifTrue: [
		self should:   (self searchCollection: [:helper | helper matchTime: #thisWeek]) isEmpty.
		self shouldnt: (self searchCollection: [:helper | helper matchTime: #lastWeek]) isEmpty]
	ifFalse: [
		self shouldnt: (self searchCollection: [:helper | helper matchTime: #thisWeek]) isEmpty.
		self should:   (self searchCollection: [:helper | helper matchTime: #lastWeek]) isEmpty].
	TimeStamp now day = 1 ifTrue: [
		self should:   (self searchCollection: [:helper | helper matchTime: #thisMonth]) isEmpty.
		self shouldnt: (self searchCollection: [:helper | helper matchTime: #lastMonth]) isEmpty]
	ifFalse: [
		self shouldnt: (self searchCollection: [:helper | helper matchTime: #thisMonth]) isEmpty.
		self should:   (self searchCollection: [:helper | helper matchTime: #lastMonth]) isEmpty].! !

TestCase subclass: #XpfCtTextTest
	instanceVariableNames: 'sut att1 att2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Tests'!

!XpfCtTextTest methodsFor: 'running' stamp: 'mt 5/5/2009 15:06'!
setUp
	
	super setUp.
	
	sut := 'The quick brown fox jumps over the lazy dog.' asText.
	att1 := XpfComplexAttribute forTag: #talk.
	att2 := XpfComplexAttribute forTag: #bad.! !

!XpfCtTextTest methodsFor: 'tests' stamp: 'mt 5/2/2009 10:19'!
testAttributesOfKind

	| atts |
	sut addAttribute: att1 from: 1 to: 5.
	sut addAttribute: att2 from: 6 to: 10.
	
	atts := sut attributesOfKind: att1 class.
	
	self
		should: atts size = 2;
		should: atts first == att1;
		should: atts second == att2.! !

!XpfCtTextTest methodsFor: 'tests' stamp: 'mt 5/18/2009 11:14'!
testAttributesOfKindFromTo

	| atts1 atts2 atts3 atts4 atts5 |
	sut addAttribute: att1 from: 1 to: 5.
	sut addAttribute: att2 from: 6 to: 10.
	
	atts1 := sut attributesOfKind: att1 class from: 1 to: 5.
	atts2 := sut attributesOfKind: att2 class from: 6 to: 10.
	atts3 := sut attributesOfKind: att1 class from: 1 to: 10.
	atts4 := sut attributesOfKind: att1 class from: 5 to: 5.
	atts5 := sut attributesOfKind: att1 class from: 5 to: 4.
	
	self
		should: atts1 size = 1;
		should: atts2 size = 1;
		should: atts3 size = 2;
		should: atts4 size = 1;
		should: atts5 size = 1;
		should: atts1 first == att1;
		should: atts2 first == att2;
		should: atts3 first == att1;
		should: atts3 second == att2;
		should: atts4 first == att1;
		should: atts5 first == att1.! !

!XpfCtTextTest methodsFor: 'tests' stamp: 'mt 5/2/2009 10:24'!
testIncludesAttributesOfKind

	sut addAttribute: att1 from: 1 to: 1.
	self should: (sut includesAttributesOfKind: att1 class).! !

!XpfCtTextTest methodsFor: 'tests' stamp: 'mt 5/3/2009 18:35'!
testIncludesAttributesOfKindFromTo

	sut addAttribute: att1 from: 1 to: 2.
	self
		should: (sut includesAttributesOfKind: att1 class from: 1 to: 2);
		shouldnt: (sut includesAttributesOfKind: att1 class from: 3 to: 4).! !

!XpfCtTextTest methodsFor: 'tests' stamp: 'mt 5/2/2009 10:27'!
testRemoveAttribute

	sut addAttribute: att1 from: 1 to: 5.
	sut addAttribute: att2 from: 6 to: 10.
	sut removeAttribute: att1.
		
	self
		should: (sut includesAttributesOfKind: att1 class);
		shouldnt: ((sut attributesOfKind: att1 class) includes: att1);
		should: ((sut attributesOfKind: att1 class) includes: att2).! !

!XpfCtTextTest methodsFor: 'tests' stamp: 'mt 5/2/2009 10:28'!
testRemoveAttributeAround

	sut addAttribute: att1 from: 1 to: 5.
	sut addAttribute: att2 from: 6 to: 10.
	sut removeAttribute: att1 around: 5.
	sut removeAttribute: att2 around: 5.
		
	self
		should: (sut includesAttributesOfKind: att1 class);
		shouldnt: ((sut attributesOfKind: att1 class) includes: att1);
		should: ((sut attributesOfKind: att1 class) includes: att2).! !

!XpfCtTextTest methodsFor: 'tests' stamp: 'mt 5/2/2009 10:30'!
testRemoveAttributeAroundAnd

	sut addAttribute: att1 from: 1 to: 5.
	sut addAttribute: att2 from: 6 to: 10.
	sut removeAttribute: att1 around: 4 and: 6.
		
	self
		should: (sut includesAttributesOfKind: att1 class);
		shouldnt: ((sut attributesOfKind: att1 class) includes: att1);
		should: ((sut attributesOfKind: att1 class) includes: att2).! !

!XpfCtTextTest methodsFor: 'tests' stamp: 'mt 5/2/2009 10:32'!
testRemoveAttributeFromTo

	sut addAttribute: att1 from: 1 to: 5.
	sut addAttribute: att2 from: 6 to: 10.
	sut removeAttribute: att1 from: 1 to: 5.
	sut removeAttribute: att2 from: 6 to: 9.
		
	self
		should: (sut includesAttributesOfKind: att1 class);
		shouldnt: ((sut attributesOfKind: att1 class) includes: att1);
		should: ((sut attributesOfKind: att1 class) includes: att2).! !

!OBTextSelection methodsFor: '*xpf-codetalk' stamp: 'mt 5/3/2009 18:39'!
start

	^ selection first! !

!OBTextSelection methodsFor: '*xpf-codetalk' stamp: 'mt 5/3/2009 18:39'!
stop

	^ selection last! !

PanelMorph subclass: #XpfCtMessageMorph
	instanceVariableNames: 'timestamp author message'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Morphs'!

!XpfCtMessageMorph class methodsFor: 'instance creation' stamp: 'mt 5/3/2009 10:53'!
timestamp: aTimeStamp author: aString message: messageString

	^ self new
		author: aString;
		timestampContent: aTimeStamp;
		messageContent: messageString! !

!XpfCtMessageMorph methodsFor: 'accessing' stamp: 'mt 4/30/2009 09:03'!
author

	author ifNil: [author := ''].
	^ author! !

!XpfCtMessageMorph methodsFor: 'accessing' stamp: 'mt 4/30/2009 09:04'!
author: aString

	author := aString! !

!XpfCtMessageMorph methodsFor: 'accessing' stamp: 'mt 5/3/2009 14:28'!
highlightIf: aString
	"Change backgroundcolor if the string is in the message."
	
	(self message contents asString asLowercase includesSubString: aString asLowercase)
		ifTrue: [self color: Color yellow].! !

!XpfCtMessageMorph methodsFor: 'initialization' stamp: 'mt 5/2/2009 18:04'!
initialize

	super initialize.
	self
		changeTableLayout;
		hResizing: #spaceFill;
		vResizing: #shrinkWrap;
		listDirection: #topToBottom;
		borderWidth: 0;
		layoutInset: 2;
		cellInset: 0;
		fillStyle: Color white;
		width: XpfCtCodeChatMorph defaultExtent x;
		initializeUserInterface.! !

!XpfCtMessageMorph methodsFor: 'initialization' stamp: 'mt 4/30/2009 09:04'!
initializeUserInterface

	self
		addMorphBack: self timestamp;
		addMorphBack: self message.! !

!XpfCtMessageMorph methodsFor: 'accessing' stamp: 'mt 5/2/2009 11:33'!
message

	message ifNil: [message := TextMorph new
		hResizing: #spaceFill;
		vResizing: #rigid;
		lock].
	^ message! !

!XpfCtMessageMorph methodsFor: 'accessing' stamp: 'mt 5/3/2009 10:51'!
messageContent: aString

	self message contentsWrapped: aString asText.
	self message font: (XpfClient sansFontAt: 10).! !

!XpfCtMessageMorph methodsFor: 'accessing' stamp: 'mt 4/30/2009 09:24'!
timestamp

	timestamp ifNil: [timestamp := TextMorph new
		hResizing: #spaceFill;
		vResizing: #rigid;
		lock].
	^ timestamp! !

!XpfCtMessageMorph methodsFor: 'accessing' stamp: 'mt 5/3/2009 10:52'!
timestampContent: aTimeStamp
	"Add the given timestamp and the authors initials to the text entry."
	
	| text |
	text := '{1} - {2}:' format: {aTimeStamp printRelativeTimeStamp. self author}.
	self timestamp contents: text.
	self timestamp font: ((XpfClient sansFontAt: 8) emphasized: (TextEmphasis bold emphasisCode)).
	! !

PanelMorph subclass: #XpfCtRowMorph
	instanceVariableNames: 'lastPosition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Core'!

!XpfCtRowMorph class methodsFor: 'as yet unclassified' stamp: 'mt 5/1/2009 17:18'!
for: someMorphs

	| instance |
	instance := self new.
	someMorphs do: [:m | instance addMorphBack: m].
	^ instance! !

!XpfCtRowMorph methodsFor: 'accessing' stamp: 'mt 5/2/2009 14:12'!
initialize

	super initialize.
	self
		changeTableLayout;
		listDirection: #leftToRight;
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap;
		layoutInset: 3;
		useRoundedCorners;
		cellInset: 3;
		color: (Color r: 4 g: 133 b: 157 alpha: 0.9).! !

!XpfCtRowMorph methodsFor: 'accessing' stamp: 'mt 5/2/2009 14:12'!
position: aPoint

	aPoint = lastPosition ifTrue: [^ self].
	lastPosition := aPoint.
	super position: aPoint.! !

OBTextMorphEditorWithShout subclass: #XpfEditor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Core'!

!XpfEditor class methodsFor: 'initialization' stamp: 'mt 5/5/2009 15:17'!
initializeCmdKeyShortcuts

	super initializeCmdKeyShortcuts.
	CmdActions
		at: ($t asInteger + 1) put: #justTalk:;
		at: ($r asInteger + 1) put: #markAsBad:.! !

!XpfEditor methodsFor: 'code markup' stamp: 'mt 5/18/2009 10:53'!
changeMarkupTag

	| result |
	result := UITheme current
		chooseDropListIn: self morph
		text: 'Select a new tag:'
		title: 'Change Markup Tag' list: XpfComplexAttribute tags.
		
	result ifNil: [^ false].
	
	(paragraph changeTagInSelectionTo: result)
		ifTrue: [self userHasEdited. ^ true].
	^ false! !

!XpfEditor methodsFor: 'initialize-release' stamp: 'cs 5/2/2009 13:44'!
changeParagraph: aParagraph
	"Add a back reference to the textmorph to the paragraph."

	super changeParagraph: aParagraph.
	(aParagraph isKindOf: XpfParagraph) & (morph isKindOf: XpfTextMorph)
		ifTrue: [
			aParagraph textMorph: morph.
			morph removeAllMorphs.
			(aParagraph text attributesOfKind: XpfComplexAttribute)
				do: [:attr | attr addDependent: morph]].! !

!XpfEditor methodsFor: 'code markup' stamp: 'mt 5/5/2009 15:02'!
justTalk: aCharacterStream

	(paragraph tagSelectionWith: #talk)
		ifTrue: [self userHasEdited. ^ true].
	^ false! !

!XpfEditor methodsFor: 'code markup' stamp: 'mt 5/1/2009 15:59'!
markAsBad: aCharacterStream

	(paragraph tagSelectionWith: #bad)
		ifTrue: [self userHasEdited. ^ true].
	^ false! !

!XpfEditor methodsFor: 'code markup' stamp: 'mt 5/1/2009 14:55'!
removeMarkup: aCharacterStream

	paragraph untagSelection.
	self userHasEdited.
	^ true! !

!XpfEditor methodsFor: 'code markup' stamp: 'mt 5/5/2009 15:02'!
resizeMarkup: aCharacterStream

	(paragraph resizeTagInSelection)
		ifTrue: [self userHasEdited. ^ true].
	^ false! !

!OBMethodVersion methodsFor: '*xpf-codetalk-override' stamp: 'mt 5/3/2009 09:39'!
source

	| file |
	file := sources at: (sources fileIndexFromSourcePointer: pointer).
	file position: (sources filePositionFromSourcePointer: pointer).
	^ file nextChunkText makeSelectorBold! !

SHTextStylerST80 subclass: #XpfTextStyler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Core'!

!XpfTextStyler methodsFor: 'private' stamp: 'mt 4/29/2009 23:01'!
privateStyle: aText
	"First, backup current style information about background color in text runs. Then let Shout work. Then restore background color information for text runs."

	| backupRuns |
	backupRuns := aText runs.

	super privateStyle: aText.
	
	backupRuns withStartStopAndValueDo: [:start :stop :attrs | |a|
		a := attrs detect: [:attr | attr isKindOf: XpfComplexAttribute] ifNone: [nil].
		a ifNotNil: [aText addAttribute: a from: start to: stop].
	]! !

!XpfTextStyler methodsFor: 'converting' stamp: 'mt 4/29/2009 23:02'!
unstyledTextFrom: aText

	| backupRuns result |
	backupRuns := aText runs.

	result := super unstyledTextFrom: aText.
	
	backupRuns withStartStopAndValueDo: [:start :stop :attrs | |a|
		a := attrs detect: [:attr | attr isKindOf: XpfComplexAttribute] ifNone: [nil].
		a ifNotNil: [result addAttribute: a from: start to: stop]].
	^ result! !

OBFilter subclass: #XpfCtReloadFilter
	instanceVariableNames: 'inspector cache announcer node isReloading browser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Versions'!

!XpfCtReloadFilter methodsFor: 'accessing' stamp: 'mt 5/25/2009 11:09'!
announcer

	^ announcer! !

!XpfCtReloadFilter methodsFor: 'accessing' stamp: 'mt 5/25/2009 12:45'!
announcer: anObject

	announcer := anObject.! !

!XpfCtReloadFilter methodsFor: 'accessing' stamp: 'mt 5/25/2009 11:48'!
browser

	^ browser! !

!XpfCtReloadFilter methodsFor: 'accessing' stamp: 'mt 5/25/2009 11:48'!
browser: anObject

	browser := anObject.! !

!XpfCtReloadFilter methodsFor: 'accessing' stamp: 'mt 5/25/2009 11:22'!
cache

	cache ifNil: [
		self cache: OrderedCollection new].
	^ cache! !

!XpfCtReloadFilter methodsFor: 'accessing' stamp: 'mt 5/19/2009 16:45'!
cache: anObject

	cache := anObject.! !

!XpfCtReloadFilter methodsFor: 'as yet unclassified' stamp: 'mt 5/25/2009 15:32'!
initializeForNode: aMethodNode

	| definition |
	definition := aMethodNode definition.
	self inspector: (XpfCtMczInspector newForPackageNamed: 
		(PackageOrganizer default packageOfClass: definition theClass) packageName).
		
	node := aMethodNode.
		
	self inspector
		addDependent: self;
		inspectVersionsFor: definition selector
		in: definition theClass.! !

!XpfCtReloadFilter methodsFor: 'accessing' stamp: 'mt 5/19/2009 16:47'!
inspector

	^ inspector! !

!XpfCtReloadFilter methodsFor: 'accessing' stamp: 'mt 5/19/2009 16:45'!
inspector: anObject

	inspector := anObject.! !

!XpfCtReloadFilter methodsFor: 'as yet unclassified' stamp: 'mt 5/25/2009 10:47'!
nodesFrom: targetNodes forNode: sourceNode

	Transcript show: 'Passing filter...'; cr.
	^ targetNodes! !

!XpfCtReloadFilter methodsFor: 'as yet unclassified' stamp: 'mt 5/25/2009 12:45'!
update: aSymbol with: anObject

	| sel |
	super update: aSymbol with: anObject.
	
	aSymbol = #versionFound ifTrue: [
		Transcript show: 'Cache updated.'; cr.
		self cache add: (XpfCtRemoteMethodVersion
			repository: (anObject at: #repository)
			info: (anObject at: #info)
			definition: (anObject at: #definition)).
		"self announcer announce: (OBChildrenChanged node: node)".


		self browser navigationPanel currentColumn ifNotNilDo: [:col |
			sel := col selection].

		self announcer announce: OBRefreshRequired new.

		sel ifNotNilDo: [:s |
			self browser navigationPanel currentColumn selection: s]].! !

!OBCodeBrowser methodsFor: '*xpf-codetalk-commands' stamp: 'mt 5/4/2009 14:42'!
cmdCodeTalk

	^ XpfCtCommand allSubclasses! !

!OBCodeBrowser methodsFor: '*xpf-codetalk-commands' stamp: 'mt 5/19/2009 15:19'!
cmdRemoteVersions
	^ XpfCtCmdBrowseRemoteMethodVersions! !

OBCodeBrowser subclass: #XpfCtRemoteVersionBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Versions'!

!XpfCtRemoteVersionBrowser class methodsFor: 'layout' stamp: 'mt 5/19/2009 10:21'!
maxPanes

	^ 1! !

!XpfCtRemoteVersionBrowser class methodsFor: 'layout' stamp: 'mt 5/19/2009 10:21'!
minPanes

	^ 1! !

!XpfCtRemoteVersionBrowser class methodsFor: 'instance creation' stamp: 'mt 5/25/2009 11:48'!
openOn: aMethodNode

	| instance version rootNode filter |
	version := OBMetaNode named: 'RemoteVersion'.	
	rootNode := OBMetaNode named: 'Method'.
	filter := XpfCtReloadFilter new.
	
	version ancestrySelector: #isDescendantOfMethodVersion:.
	
	rootNode
		childAt: #remoteVersions put: version;
		addFilter: filter.

	instance := self metaNode: rootNode root: aMethodNode selection: nil.
	
	filter
		browser: instance;
		announcer: instance announcer;
		initializeForNode: aMethodNode.

	^ instance open! !

!XpfCtRemoteVersionBrowser methodsFor: 'building' stamp: 'mt 5/25/2009 10:13'!
defaultBackgroundColor

	^ Color gray muchLighter! !

!XpfCtRemoteVersionBrowser methodsFor: 'accessing' stamp: 'mt 5/19/2009 10:21'!
defaultLabel

	^ 'Remote Versions of ', self root name printString! !

SystemProgressBarMorph subclass: #XpfCtPluggableProgressBarMorph
	instanceVariableNames: 'model maxSelector valueSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Morphs'!

!XpfCtPluggableProgressBarMorph class methodsFor: 'as yet unclassified' stamp: 'mt 5/5/2009 20:53'!
newFor: aModel value: valSel max: maxSel

	^ self new
		model: aModel;
		maxSelector: maxSel;
		valueSelector: valSel! !

!XpfCtPluggableProgressBarMorph methodsFor: 'accessing' stamp: 'mt 5/5/2009 20:45'!
maxSelector

	^ maxSelector! !

!XpfCtPluggableProgressBarMorph methodsFor: 'accessing' stamp: 'mt 5/5/2009 20:45'!
maxSelector: aSymbol

	maxSelector := aSymbol! !

!XpfCtPluggableProgressBarMorph methodsFor: 'accessing' stamp: 'mt 5/5/2009 20:42'!
model

	^ model! !

!XpfCtPluggableProgressBarMorph methodsFor: 'accessing' stamp: 'mt 5/5/2009 20:43'!
model: anObject

	self model ifNotNilDo: [:m | m removeDependent: self].
	model := anObject.
	self model ifNotNilDo: [:m | m addDependent: self].! !

!XpfCtPluggableProgressBarMorph methodsFor: 'updating' stamp: 'mt 5/5/2009 21:24'!
update: aSymbol

	super update: aSymbol.
	
	aSymbol = self valueSelector ifTrue: [self updateBar]! !

!XpfCtPluggableProgressBarMorph methodsFor: 'updating' stamp: 'mt 5/5/2009 21:26'!
updateBar

	| value max |
	value := self model perform: self valueSelector.
	max := self model perform: self maxSelector.
	
	self 
		disableTableLayout: ((value = 0) | (value = max));
		visible: ((value = 0) | (value = max)) not.	
	
	self visible ifTrue:
		[self barSize: (value * self width) // max].! !

!XpfCtPluggableProgressBarMorph methodsFor: 'accessing' stamp: 'mt 5/5/2009 20:45'!
valueSelector

	^ valueSelector! !

!XpfCtPluggableProgressBarMorph methodsFor: 'accessing' stamp: 'mt 5/5/2009 20:54'!
valueSelector: aSymbol

	valueSelector := aSymbol.
	self update: aSymbol.! !

!OBMethodVersionNode methodsFor: '*xpf-codetalk' stamp: 'mt 5/3/2009 10:02'!
categoryText

	^ self version category! !

!OBMethodVersionNode methodsFor: '*xpf-codetalk' stamp: 'mt 5/3/2009 09:55'!
sourceText

	^ self version source! !

OBMethodVersionNode subclass: #XpfCtRemoteMethodVersionNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Versions'!

!XpfCtRemoteMethodVersionNode class methodsFor: 'as yet unclassified' stamp: 'mt 5/19/2009 11:20'!
on: aRemoteMethodVersion

	^ self new setVersion: aRemoteMethodVersion! !

!XpfCtRemoteMethodVersionNode methodsFor: 'accessing' stamp: 'mt 5/19/2009 15:31'!
remoteVersion

	^ version! !

!XpfCtRemoteMethodVersionNode methodsFor: 'accessing' stamp: 'mt 5/19/2009 15:31'!
remoteVersion: anObject

	version := anObject.! !

!XpfCtRemoteMethodVersionNode methodsFor: 'as yet unclassified' stamp: 'mt 5/19/2009 11:28'!
source

	^ self sourceText! !

!XpfCtRemoteMethodVersionNode methodsFor: 'as yet unclassified' stamp: 'mt 5/19/2009 11:26'!
sourceText

	^ self remoteVersion sourceText! !

!XpfCtRemoteMethodVersionNode methodsFor: 'as yet unclassified' stamp: 'mt 5/19/2009 15:35'!
theClass

	^ self version theClass! !

!MenuIcons class methodsFor: '*xpf-codetalk' stamp: 'mt 2/5/2009 18:08'!
markBadIcon
	^ Icons
			at: #'markBad'
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self markBadIconContents readStream) ].! !

!MenuIcons class methodsFor: '*xpf-codetalk' stamp: 'mt 2/5/2009 18:06'!
markBadIconContents
	^ 'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0
RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAACxSURBVDjL7dItC8JQAIXh491H
WbDvD1pEDIIKm1zDxqKi0WIULAZFYc1msF3ErEZlMC9Mhxx/gTAR28KpDxx4QRK/DCXwAbDg
0oLMTShtQF0F5AlwvwHkqy+Zxxs+lwsmvs8DKrIw8DCh8njNLOxRtxrU4yF3MFRhIBFQ2XxG
3W7yXq8xjULGsIsDFwF58zzq0YBpFPDc6XIKp/iFI4S7h5BbWGoFW03gyABVtyzxT8Ab8Xze
i+tkcykAAAAASUVORK5CYII='! !

!MenuIcons class methodsFor: '*xpf-codetalk' stamp: 'mt 2/5/2009 18:08'!
markGoodIcon
	^ Icons
			at: #'markGood'
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self markGoodIconContents readStream) ].! !

!MenuIcons class methodsFor: '*xpf-codetalk' stamp: 'mt 2/5/2009 18:06'!
markGoodIconContents
	^ 'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0
RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAC5SURBVDjLY/j//z8DJZhh1ADs
BjjsspIC4gb77ZZX7TdbXLVda9Zgs8xEihQDGmZfm/7/0KOD/3ff3/V/6plJ/y3mGjYQbYD9
NsurBx4e+D/10tT/nWc6/i+5sui/+RS9q0QbYLfB/OrWO1v+d5/p+t96qvn/3Auz/pt0aRNv
gPVyk4appyf+X3xl4f/ZF2b+n3Co579+mSrxXrCcZyhlMV2/wbRP56pRq+ZV3SLlBq1EOanR
lEgjAwAXIuIDq5qm/AAAAABJRU5ErkJggg=='! !

!MenuIcons class methodsFor: '*xpf-codetalk' stamp: 'mt 2/5/2009 18:08'!
markUnknownIcon
	^ Icons
			at: #'markUnknown'
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self markUnknownIconContents readStream) ].! !

!MenuIcons class methodsFor: '*xpf-codetalk' stamp: 'mt 2/5/2009 18:07'!
markUnknownIconContents
	^ 'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAQAAAC1+jfqAAAABGdBTUEAAK/INwWK6QAAABl0
RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAABlSURBVCjPY/jPgB8yDC4FilKK
DfJXZa6KNwhKYVfQkPW/63/b/+T/XA1YFchd7fqf/j/2f+N/5qtYFUhe7fif9D/sf91/BuwK
hBoS/jcBpcP/M2C3gluKrYHhKhA2MEgN2pDEDgEb0f5zlvXgVgAAAABJRU5ErkJggg=='! !

!MenuIcons class methodsFor: '*xpf-codetalk' stamp: 'mt 2/5/2009 18:08'!
removeMarkupIcon
	^ Icons
			at: #'removeMarkup'
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self removeMarkupIconContents readStream) ].! !

!MenuIcons class methodsFor: '*xpf-codetalk' stamp: 'mt 2/5/2009 18:07'!
removeMarkupIconContents
	^ 'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAQAAAC1+jfqAAAABGdBTUEAAK/INwWK6QAAABl0
RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAABbSURBVCjPzdAxDoAgEERRzsFp
95JbGI2ASA2SCOX3Ahtr8tuXTDIO959bCxRfpOitWS5vA+lMJg9JbKCTTmMQ1QS3ThqVQbBB
lsbgpXLYE8lHCXrqLptf9km7Dzv+FwGTaznIAAAAAElFTkSuQmCC'! !

!MenuIcons class methodsFor: '*xpf-codetalk' stamp: 'mt 5/1/2009 17:52'!
xpfCtCommentAddContents
	^ 'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0
RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAGkSURBVDjLrZJPKINhHMd3UsrJ
TcpBDi6UC+3ookQ5OOBAaCh2cFQ40GqTzURTihI54DRp8dqBg3BQw5BtNmxe/4flT9re5+v3
vJvF2l6Kp749Pe/7fj7P7/09jwqA6i9R/ZugVr+cSZmlvFOgEIGSl0xgnVt3IRyRoDSWtn1c
4qakxQW0yKBEJMbw+MpwHWIQnxgCDwxnQQbvHYP7RoLnJirvntrkkuKvghytZU1+eUWg+Mjg
J/j0nuEkBh9dSTgQo4KB+R0uqEgquCD4PBiDbxlc11HYSfBuILUg/gu8fB/t6rmVcEzw4aWE
fYIdAS6IyILe6S0uUCdtIpd8Hbwah1+SxQlNTE91jJHPI5tcPoiLrBsL6BxrQOtQFep0pc/l
XYU9P14kkngugy/onxlF30ITlpwWOEQB5tV21JgLUNKRZVSCTeM2J6/kuV5fFrbuD8N6OCJX
Y7S3wGxv44K3VHAuxUvR8HVldxFszolvvVncs3DB7+67Wpv9Nig0Qy80yrB+pVG5gsTQh7pq
Yz5Mgkbemc98rdiDJBIDJcTLjs0G/vwDCw/6dFwBuzsAAAAASUVORK5CYII='! !

!MenuIcons class methodsFor: '*xpf-codetalk' stamp: 'mt 5/1/2009 17:53'!
xpfCtCommentAddIcon
	^ Icons
			at: #'xpfCtCommentAdd'
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self xpfCtCommentAddContents readStream) ].! !

!MenuIcons class methodsFor: '*xpf-codetalk' stamp: 'mt 5/1/2009 17:51'!
xpfCtCommentsContents
	^ 'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0
RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAG/SURBVDjLjZK9T8JQFMVZTUyc
3IyJg4mDi87+GyYu6qB/gcZdFxkkJM66qJMGSNRBxDzigJMRQ1jQ4EcQ+SgVKB+FtuL13EdJ
xNDq8Ev7Xu85797T51nwhqeAH5w6cAxWwDgReX7jwYfdaCIraroptB7NLlVQrOoiGEsL1G06
GZyxuILicsMUH3VTlOqGKNUMUdTacj+j1Nng0NGAT2WxYosK1bbIVVoiW27J9V8G57WWKVSc
zMV5iK+Tudv1vVh5yXdlLQN+os4AFZss2Ob82CCgQmhYHSnmkzf2b6rIhTAaaT2aXZALIRdC
LgRtkA1WfYG4iKcVYX52JIs7EYvFmJ8wGiEXQi6EXAhdyn2MxQaPcg68zIETTvzyLsPzWnwq
ixVbhFwI3RFykes+A9vkIBKX4jCoIxdCLrI4/0OcUXXK4/1dbbDBS088xGGCCzAJCsiF2lan
T8xdKNhHXvRarLFBqmcwCrbAhL32+kP3lHguETKRsNlbqUFPeY2OoikW62DNM+jf2ibzQNN0
g5ALC75AGiT59oIReQ+cDGyTB+TC4jaYGXiRXMTD3AFogVmnOjeDMRAC025duo7wH74BwZ8J
lHrTPLcAAAAASUVORK5CYII='! !

!MenuIcons class methodsFor: '*xpf-codetalk' stamp: 'mt 5/1/2009 17:54'!
xpfCtCommentsIcon
	^ Icons
			at: #'xpfCtComments'
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self xpfCtCommentsContents readStream) ].! !

!MenuIcons class methodsFor: '*xpf-codetalk' stamp: 'mt 5/1/2009 17:52'!
xpfCtCrossContents
	^ 'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0
RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAIhSURBVDjLlZPrThNRFIWJicmJ
z6BWiYbIkYDEG0JbBiitDQgm0PuFXqSAtKXtpE2hNuoPTXwSnwtExd6w0pl2OtPlrphKLSXh
x07OZM769qy19wwAGLhM1ddC184+d18QMzoq3lfsD3LZ7Y3XbE5DL6Atzuyilc5Ciyd7IHVf
gNcDYTQ2tvDr5crn6uLSvX+Av2Lk36FFpSVENDe3OxDZu8apO5rROJDLo30+Nlvj5RnTlVNA
Ks1aCVFr7b4BPn6Cls21AWgEQlz2+Dl1h7IdA+i97A/geP65WhbmrnZZ0GIJpr6OqZqYAd5/
gJpKox4Mg7pD2YoC2b0/54rJQuJZdm6Izcgma4TW1WZ0h+y8BfbyJMwBmSxkjw+VObNanp5h
/adwGhaTXF4NWbLj9gEONyCmUZmd10pGgf1/vwcgOT3tUQE0DdicwIod2EmSbwsKE1P8QoDk
cHPJ5YESjgBJkYQpIEZ2KEB51Y6y3ojvY+P8XEDN7uKS0w0ltA7QGCWHCxSWWpwyaCeLy0Bk
A7UXyyg8fIzDoWHeBaDN4tQdSvAVdU1Aok+nsNTipIEVnkywo/FHatVkBoIhnFisOBoZxcGt
Qd4B0GYJNZsDSiAEadUBCkstPtN3Avs2Msa+Dt9XfxoFSNYF/Bh9gP0bOqHLAm2WUF1YQskw
rVFYPWkf3h1iXwbvqGfFPSGW9Eah8HSS9fuZDnS32f71m8KFY7xs/QZyu6TH2+2+FAAAAABJ
RU5ErkJggg=='! !

!MenuIcons class methodsFor: '*xpf-codetalk' stamp: 'mt 5/1/2009 17:54'!
xpfCtCrossIcon
	^ Icons
			at: #'xpfCtCross'
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self xpfCtCrossContents readStream) ].! !

!MenuIcons class methodsFor: '*xpf-codetalk' stamp: 'mt 5/1/2009 17:52'!
xpfCtDeleteContents
	^ 'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0
RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAJdSURBVDjLpZP7S1NhGMf9W7Yf
ogSJboSEUVCY8zJ31trcps6zTI9bLGJpjp1hmkGNxVz4Q6ildtXKXzJNbJRaRmrXoeWx8tJO
TWptnrNryre5YCYuI3rh+8vL+/m8PA/PkwIg5X+y5mJWrxfOUBXm91QZM6UluUmthntHqplx
Uml2lciF6wrmdHriI0Wx3xw2hAediLwZRWRkCPzdDswaSvGqkGCfq8VEUsEyPF1O8Qu3O7A0
9RbRvjuIttsRbT6HHzebsDjcB4/JgFFlNv9MnkmsEszodIIY7Oaut2OJcSF68Qx8dgv8tmqE
L1gQaaARtp5A+N4NzB0lMXxon/uxbI8gIYjB9HytGYuusfiPIQcN71kjgnW6VeFOkgh3XcHL
vAwMSDPohOADdYQJdF1FtLMZPmslvhZJk2ahkgRvq4HHUoWHRDqTEDDl2mDkfheiDgt8pw34
0/EocuClCuFvboQzb0cwIZgki4KhzlaE6w0InipbVzBfqoK/qRH94i0rgokSFeO11iBkp8Ed
V8cfJo0yD75aE2ZNRvSJ0lZKcBXLaUYmQrCzDT6tDN5SyRqYlWeDLZAg0H4JQ+Jt6M3atNLE
10VSwQsN4Z6r0CBwqzXesHmV+BeoyAUri8EyMfi2FowXS5dhd7doo2DVII0V5BAjigP89GEV
Atda8b2ehodU4rNaAW+dGfzlFkyo89GTlcrHYCLpKD+V7yeeHNzLjkp24Uu1Ed6G8/F8qjqG
Rzlbl2H2dzjpMg1KdwsHxOlmJ7GTeZC/nesXbeZ6c9OYnuxUc3fmBuFft/Ff8xMd0s65SXIb
/gAAAABJRU5ErkJggg=='! !

!MenuIcons class methodsFor: '*xpf-codetalk' stamp: 'mt 5/1/2009 17:54'!
xpfCtDeleteIcon
	^ Icons
			at: #'xpfCtDelete'
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self xpfCtDeleteContents readStream) ].! !

!MenuIcons class methodsFor: '*xpf-codetalk' stamp: 'mt 5/3/2009 14:11'!
xpfCtMagnifierContents
	^ 'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0
RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAH5SURBVDjLpZK/a5NhEMe/748k
RqypmqQQgz/oUPUPECpCoEVwyNStIA6COFR33boIjg6mg4uL0k0EO1RFISKImkHQxlbQRAsx
0dgKJm/e53nunnOwViR5leJnuZs+973jHBHB/+D/ah7X2LXWloilyMw5YgtD3CDiBWN4Zno8
bQcJHBFBucauZfsolZDCru0OfFcAAUISrLZDfPzSKxuiibOT+T6JCwDMtrQzYQvZHQ5Cw2h3
GK0OI9AWBzJJZFOxgtJUGpTABQAiLu5OOviuGIEWkBUwC7pasNZj7N2ThNJUjBQY4pznAoEW
sBWwxU+JFXSVRTzmQWvKRR5RG4KVGMgKrAVYflexAAugDCEygdbUCI2F7zobk7FZY76DIDQg
rT9HCwwt1FsBhhIu4p4D3kiS8B0MJz28ftfGSPfl8MPLxbGBAqVpptbslJc+fEPMA7JDPrIp
H3FX8LzaROdrE5O51jalgid3Lh4b6/sDALh6971riErGcFET58gwDPGndG9JT6ReHcwfPorG
ygu8rdxvGxMeP3XtzcofgigWZ0/EtQ7n0/sOTe0/Mo7V5WeoVu61z1yvZzZX+BsnZx9opYLp
evXp7eXKIrL5UWit0n0r/Isb50bjRGreiyWmgs76lfM31y5tSQAAc6czHjONXLi13thygih+
AEq4N6GqMsuhAAAAAElFTkSuQmCC'! !

!MenuIcons class methodsFor: '*xpf-codetalk' stamp: 'mt 5/3/2009 14:11'!
xpfCtMagnifierIcon
	^ Icons
			at: #'xpfCtMagnifier'
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self xpfCtMagnifierContents readStream) ].! !

!MenuIcons class methodsFor: '*xpf-codetalk' stamp: 'mt 5/5/2009 14:42'!
xpfCtMarkBadIcon

	^ Icons
			at: #'xpfCtMarkBad'
			ifAbsentPut:[ UIThemeWatery new newWindowCloseForm ].! !

!MenuIcons class methodsFor: '*xpf-codetalk' stamp: 'mt 5/5/2009 14:41'!
xpfCtMarkCommentIcon

	^ Icons
			at: #'xpfCtMarkComment'
			ifAbsentPut:[ UIThemeWatery new newWindowMinimizeForm ].! !

!MenuIcons class methodsFor: '*xpf-codetalk' stamp: 'mt 5/3/2009 19:16'!
xpfCtPaintcanContents
	^ 'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0
RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAJVSURBVDjLlZPbS5NhHMd3tbvy
ov6HIJRuhKAFgRci5GXYVQkG0c6HJGvD1FVMjdxC22wlQ9iG7uBh0rtDmxtb77vFoh2c7KgM
tiGbU9xdgezb3hf0Ii3cxZfngef5fn6H5/ewALD+Vv8Upeh7HfzdM+pb4QwT7PPunOg8M7tl
Ph63JtE/4UM3f7WnLUDL3NE7vgmFKYbeF050Di3dbQvQSrvjzogTIm0Yt0WrePp25VNbAHcg
rLopXMOgyo/uR8v4bAnAbrdLLwSIRCLK1HYaHKEdnQ9MuHZ/Edv5MpYcHljc1K9Fgvo+uxqa
nLYEr5wBhMNhZTqdRr1ex6jOBz8VxzONC+VyGQcHB5g3rcFGZWH+lscrc9ArWdjseqzzXWLM
FEWdmnO5HEwbEYzpvmKZ+IFKpYJSqQS/3w/l+4XmlCXYnHGmMfTBOzyg9lxmkSQ5RpsbjQYK
hQKy2Syz7u/vM0Z6T5+73W44nU7IlOrmS9tPPJn3VfsmXVdZLfKbra0tVKtV7O7uYmdnh8mE
BtCwVCqFRCIBgiCg0Wggk8ko7sdNCMwJ3JogRpgSHA6HOhqN4ujoCIeHh6jVashkMkgmk4jH
4wgEApibm4NUKqVEIhH74awXrfSPbyi+dJ020Wg0vguFQtjb20M+n2eixmIxBqDX6+nIpFAo
ZMb63owHHCXhOvOMWq1W7fF4UCwWQZdFN9RqtdKRSYFAcPonrj/fGPznIKlUKvX6+jrTfZvN
BolEQvL5fPaFJ5GWXC5XGwwGiMViksfjtfcbT9Rq1gCXy/2vmdYfaGviUGKvapgAAAAASUVO
RK5CYII='! !

!MenuIcons class methodsFor: '*xpf-codetalk' stamp: 'mt 5/3/2009 19:16'!
xpfCtPaintcanIcon
	^ Icons
			at: #'xpfCtPaintcan'
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self xpfCtPaintcanContents readStream) ].! !

!MenuIcons class methodsFor: '*xpf-codetalk' stamp: 'mt 5/5/2009 14:41'!
xpfCtSmallMarkBadIcon

	^ Icons
			at: #'xpfCtSmallMarkBad'
			ifAbsentPut:[ UIThemeWatery new newWindowCloseForm scaledToSize: 8@8 ].! !

!MenuIcons class methodsFor: '*xpf-codetalk' stamp: 'mt 5/5/2009 14:42'!
xpfCtSmallMarkCommentIcon

	^ Icons
			at: #'xpfCtSmallMarkComment'
			ifAbsentPut:[ UIThemeWatery new newWindowMinimizeForm scaledToSize: 8@8 ].! !

!ByteArray class methodsFor: '*xpf-codetalk' stamp: 'mt 5/3/2009 21:24'!
fromHex: hexString 
	"Returns a byteArray from a hex string. The hex string can have
	leading zeros that will not be truncated, preserving the size of the
	byteArray for things like cyrptograpy"
	| anInteger hexSize stringSize aStringOfHex |
	aStringOfHex := (hexString copyWithoutAll: (Array with: $  with: Character cr with: Character tab)).
	stringSize := aStringOfHex size.
	(stringSize > 3
			and: [(aStringOfHex copyFrom: 1 to: 3) = '16r'])
		ifTrue: [anInteger := Number readFrom: (aStringOfHex asUppercase at: 3 put: $r;
							 yourself).
			hexSize := aStringOfHex size - 3 / 2]
		ifFalse: [anInteger := Number readFrom: aStringOfHex asUppercase base: 16.
			hexSize := aStringOfHex size / 2].
	^ anInteger asByteArrayOfSize2: hexSize asInteger! !

OBTextMorphWithShout subclass: #XpfTextMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Core'!

!XpfTextMorph methodsFor: 'submorphs-add/remove' stamp: 'mt 5/1/2009 14:13'!
addMorph: aMorph

	(self submorphs includes: aMorph)
		ifFalse: [^ super addMorph: aMorph].
	^ aMorph! !

!XpfTextMorph methodsFor: 'private' stamp: 'topa 2/13/2009 14:09'!
editorClass
	"Answer the class used to create the receiver's editor"
	
	^XpfEditor! !

!XpfTextMorph methodsFor: 'initialization' stamp: 'mt 4/30/2009 10:36'!
initialize

	super initialize.
	
	(Preferences preferenceAt: #showLineNumbers ifAbsent: [Preference new rawValue: false]) preferenceValue
		ifTrue: [self leftMargin: ((Preferences standardCodeFont widthOfString: '99') * 1.7) rounded].! !

!XpfTextMorph methodsFor: 'accessing' stamp: 'mt 4/29/2009 15:52'!
leftMargin: aNumber

	self margins: (aNumber@0 corner: 0@0).! !

!XpfTextMorph methodsFor: 'private' stamp: 'mt 4/22/2009 20:24'!
paragraphClass
	
	^ XpfParagraph! !

!XpfTextMorph methodsFor: 'submorphs-add/remove' stamp: 'mt 4/30/2009 14:38'!
removeAllMorphsBut: someMorphs

	self submorphs copy do: [:m | (someMorphs includes: m)
		ifFalse: [m delete]]! !

!XpfTextMorph methodsFor: 'private' stamp: 'mt 5/3/2009 19:46'!
update: aSymbol

	super update: aSymbol.
	aSymbol = #chatLineAdded ifTrue: [self hasUnacceptedEdits: true].
	aSymbol = #resizeRequested ifTrue: [
		self hasUnacceptedEdits:
			self paragraph resizeTagInSelection].! !

!XpfTextMorph methodsFor: 'private' stamp: 'mt 5/3/2009 19:45'!
update: aSymbol with: anAttribute

	aSymbol = #deleteRequested ifTrue: [
		self text removeAttribute: anAttribute.
		self hasUnacceptedEdits: true].
	
	super update: aSymbol with: anAttribute.! !

!XpfTextMorph methodsFor: 'submorphs-add/remove' stamp: 'mt 4/30/2009 13:21'!
updateSubmorphs: someMorphs

	self submorphs copy do: [:m |
		(someMorphs includes: m) ifFalse: [m delete]].
	someMorphs do: [:m |
		(self submorphs includes: m) ifFalse: [self addMorph: m]].! !

StandardWindow subclass: #XpfCtTagAnalysisMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Morphs'!

!XpfCtTagAnalysisMorph class methodsFor: 'defaults' stamp: 'mt 5/9/2009 17:20'!
defaultControlWidth

	^ 150! !

!XpfCtTagAnalysisMorph class methodsFor: 'defaults' stamp: 'mt 5/9/2009 17:47'!
defaultExtent

	^ 500@280! !

!XpfCtTagAnalysisMorph class methodsFor: 'opening' stamp: 'mt 5/9/2009 17:20'!
open

	^ self new openInWorld extent: self defaultExtent! !

!XpfCtTagAnalysisMorph class methodsFor: 'opening' stamp: 'mt 5/9/2009 17:20'!
prototypicalToolWindow
	"Used in flaps."
	
	^ self new extent: self defaultExtent! !

!XpfCtTagAnalysisMorph class methodsFor: 'accessing' stamp: 'mt 5/9/2009 17:20'!
title

	^ 'Markup analysis'! !

!XpfCtTagAnalysisMorph methodsFor: 'controls' stamp: 'mt 5/9/2009 17:26'!
analysisButton

	^ (self
		newButtonFor: self model
		getState: nil
		action: #startStop
		arguments: nil
		getEnabled: #canAnalyze
		getLabel: #label
		help: 'Click here to start analysis.')
		hResizing: #spaceFill! !

!XpfCtTagAnalysisMorph methodsFor: 'controls' stamp: 'mt 5/9/2009 17:46'!
diagram

	^ XpfCtDiagramMorph new
		dimensions: (0@5 corner: 100@0);
		addGrid: 10@1;
		addGraph: (XpfCtGraphMorph
			newFor: (XpfCtTestGraphModel newForInspector: self model inspector)
			color: Color red)! !

!XpfCtTagAnalysisMorph methodsFor: 'controls' stamp: 'mt 5/9/2009 17:54'!
filterEntry

	^ (self
		newTextEntryFor: self model
		getText: #filter
		setText: #filter:
		getEnabled: #isNotAnalyzing
		help: nil)
		hResizing: #rigid;
		vResizing: #rigid;
		extent: self class defaultControlWidth@(self theme textFont height + 8);
		autoAccept: true.! !

!XpfCtTagAnalysisMorph methodsFor: 'initialize-release' stamp: 'mt 5/9/2009 17:10'!
initialize

	super initialize.
	self
		addMorph: (self newDialogPanel listDirection: #leftToRight) frame: (0@0 corner: 1@1);
		model: XpfCtTagAnalysisModel new;
		initializeUserInterface.! !

!XpfCtTagAnalysisMorph methodsFor: 'initialize-release' stamp: 'mt 5/9/2009 17:48'!
initializeUserInterface

	self paneMorphs first
		cellInset: 20;
		addMorphBack: ((self newColumn: {
			self newRow: {(self newLabel: 'Repository') hResizing: #spaceFill. self repositoryEntry}.
			self newRow: {(self newLabel: 'Username') hResizing: #spaceFill. self usernameEntry}.
			self newRow: {(self newLabel: 'Password') hResizing: #spaceFill. self passwordEntry}.
			self newRow: {(self newLabel: 'Filter') hResizing: #spaceFill. self filterEntry}.
			self newToolSpacer vResizing: #spaceFill.
			self newSeparator.
			self analysisButton})
			hResizing: #shrinkWrap);
		addMorphBack: self diagram.! !

!XpfCtTagAnalysisMorph methodsFor: 'controls' stamp: 'mt 5/9/2009 17:54'!
passwordEntry

	^ (self
		newTextEntryFor: self model
		getText: #password
		setText: #password:
		getEnabled: #isNotAnalyzing
		help: nil)
		hResizing: #rigid;
		vResizing: #rigid;
		extent: self class defaultControlWidth@(self theme textFont height + 8);
		autoAccept: true.! !

!XpfCtTagAnalysisMorph methodsFor: 'controls' stamp: 'mt 5/9/2009 17:53'!
repositoryEntry

	^ (self
		newTextEntryFor: self model
		getText: #repositoryName
		setText: #repositoryName:
		getEnabled: #isNotAnalyzing
		help: 'Type name of repository in SqueakSource @ HPI.')
		hResizing: #rigid;
		vResizing: #rigid;
		extent: self class defaultControlWidth@(self theme textFont height + 8);
		autoAccept: true.! !

!XpfCtTagAnalysisMorph methodsFor: 'appearance' stamp: 'mt 5/9/2009 17:22'!
title

	^ self class title! !

!XpfCtTagAnalysisMorph methodsFor: 'controls' stamp: 'mt 5/9/2009 17:54'!
usernameEntry

	^ (self
		newTextEntryFor: self model
		getText: #username
		setText: #username:
		getEnabled: #isNotAnalyzing
		help: nil)
		hResizing: #rigid;
		vResizing: #rigid;
		extent: self class defaultControlWidth@(self theme textFont height + 8);
		autoAccept: true.! !

StandardWindow subclass: #XpfCtTagSearchMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Morphs'!

!XpfCtTagSearchMorph class methodsFor: 'defaults' stamp: 'mt 5/5/2009 19:34'!
defaultControlWidth

	^ 150! !

!XpfCtTagSearchMorph class methodsFor: 'defaults' stamp: 'mt 5/5/2009 18:53'!
defaultExtent

	^ 600@350! !

!XpfCtTagSearchMorph class methodsFor: 'opening' stamp: 'mt 5/5/2009 18:53'!
open

	^ self new openInWorld extent: self defaultExtent! !

!XpfCtTagSearchMorph class methodsFor: 'opening' stamp: 'mt 5/5/2009 18:59'!
prototypicalToolWindow
	"Used in flaps."
	
	^ self new extent: self defaultExtent! !

!XpfCtTagSearchMorph class methodsFor: 'accessing' stamp: 'cs 5/4/2009 11:55'!
title

	^ 'Markup finder'! !

!XpfCtTagSearchMorph methodsFor: 'controls' stamp: 'mt 5/5/2009 19:26'!
categoryEditor

	| pt |
	pt := PluggableTextEditorMorph new
		on: self model
		text: #category
		accept: #category:
		readSelection: #categoryLength
		menu: nil;
		font: self theme textFont;
		cornerStyle: self preferredCornerStyle;
		hResizing: #rigid;
		getEnabledSelector: #categoryEnabled;
		borderStyle: (BorderStyle inset width: 1);
		color: Color white;
		alwaysAccept: true;
		vResizing: #rigid;
		hideScrollBarsIndefinitely;
		extent: self class defaultControlWidth@(self theme textFont height + 8);
		setBalloonText: 'Enter category here';
		autoAccept: true.
	pt textMorph
		autoFit: true;
		wrapFlag: true;
		margins: (2@1 corner: 2@1).
	^ pt! !

!XpfCtTagSearchMorph methodsFor: 'layout' stamp: 'cs 5/3/2009 13:25'!
categoryRow

	^ self newRow: {
		(self newLabel: 'Category') hResizing: #spaceFill.
		self categoryEditor}! !

!XpfCtTagSearchMorph methodsFor: 'controls' stamp: 'mt 5/5/2009 19:26'!
classEditor

	| pt |
	pt := (PluggableTextEditorMorph new
		on: self model
		text: #theClass
		accept: #theClass:
		readSelection: #theClassLength
		menu: nil;
		getEnabledSelector: #theClassEnabled;
		font: self theme textFont;
		cornerStyle: self preferredCornerStyle;
		hResizing: #rigid;
		borderStyle: (BorderStyle inset width: 1);
		color: Color white)
		alwaysAccept: true;
		vResizing: #rigid;
		hideScrollBarsIndefinitely;
		extent: self class defaultControlWidth@(self theme textFont height + 8);
		setBalloonText: 'Enter class name here';
		autoAccept: true.
	pt textMorph
		autoFit: true;
		wrapFlag: true;
		margins: (2@1 corner: 2@1).
	^ pt! !

!XpfCtTagSearchMorph methodsFor: 'layout' stamp: 'cs 5/3/2009 12:37'!
classRow

	^ self newRow: {
		(self newLabel: 'Class') hResizing: #spaceFill.
		self classEditor}! !

!XpfCtTagSearchMorph methodsFor: 'layout' stamp: 'mt 5/5/2009 19:25'!
creatorRow

	^ self newRow: {
		(self newLabel: 'Creator') hResizing: #spaceFill.
		(self
			newTextEntryFor: self model
			getText: #creator
			setText: #creator:
			help: 'Enter the initials of the tag''s creator')
			autoAccept: true;
			width: self class defaultControlWidth;
			hResizing: #rigid}! !

!XpfCtTagSearchMorph methodsFor: 'initialization' stamp: 'cs 5/4/2009 11:07'!
initialize

	super initialize.
	self
		addMorph: (self newDialogPanel listDirection: #leftToRight) frame: (0@0 corner: 1@1);
		model: XpfCtTagSearchModel new;
		initializeUserInterface.! !

!XpfCtTagSearchMorph methodsFor: 'initialization' stamp: 'mt 5/6/2009 10:22'!
initializeUserInterface

	self paneMorphs first
		addMorphBack: ((self newColumn: {
			self categoryRow.
			self classRow.
			self tagRow.
			self creatorRow.
			self timeRow.
			self keywordRow.
			self newToolSpacer vResizing: #spaceFill.
			self newSeparator.
			self progressLabel.
			self progressBar.
			self searchButton})
			hResizing: #shrinkWrap);
		addMorphBack: self resultsList.! !

!XpfCtTagSearchMorph methodsFor: 'layout' stamp: 'mt 5/5/2009 19:25'!
keywordRow

	^ self newRow: {
		(self newLabel: 'Keyword') hResizing: #spaceFill.
		(self
			newTextEntryFor: self model
			getText: #keyword
			setText: #keyword:
			help: 'Enter a keyword to search in tag discussions')
			autoAccept: true;
			width: self class defaultControlWidth;
			hResizing: #rigid}! !

!XpfCtTagSearchMorph methodsFor: 'controls' stamp: 'mt 5/5/2009 20:57'!
progressBar

	^ (XpfCtPluggableProgressBarMorph 
		newFor: self model
		value: #progressValue
		max: #progressValueMax)
		hResizing: #spaceFill;
		vResizing: #rigid;
		height: 16! !

!XpfCtTagSearchMorph methodsFor: 'controls' stamp: 'mt 5/6/2009 10:22'!
progressLabel

	^ XpfCtPluggableLabelMorph new
		model: self model;
		labelSelector: #progressString;
		hResizing: #spaceFill! !

!XpfCtTagSearchMorph methodsFor: 'controls' stamp: 'mt 5/5/2009 20:36'!
resultsList

	^ (XpfPluggableListMorph new
			selectionColor: self theme selectionColor;
			font: self theme listFont;
			on: self model
			list: #results
			selected: nil
			changeSelected: #selectResult:
			menu: nil
			keystroke: nil)
			autoDeselect: false;
			cornerStyle: self preferredCornerStyle;
			color: Color white;
			borderStyle: (BorderStyle inset width: 1);
			hResizing: #spaceFill;
			vResizing: #spaceFill;
			setBalloonText: 'Displays search results';
			minWidth: 200! !

!XpfCtTagSearchMorph methodsFor: 'controls' stamp: 'mt 5/5/2009 21:36'!
searchButton

	^ (self
		newButtonFor: self model
		getState: nil
		action: #searchOrStop
		arguments: nil
		getEnabled: #canSearch
		getLabel: #searchButtonLabel
		help: 'Click here to search')
		hResizing: #spaceFill! !

!XpfCtTagSearchMorph methodsFor: 'layout' stamp: 'mt 5/5/2009 19:25'!
tagRow

	| chkCol |
	chkCol := (self newColumn: {})
		width: self class defaultControlWidth;
		hResizing: #rigid.
	
	XpfComplexAttribute tags do: [:tag |
		chkCol addMorphBack:  (self
			newCheckboxFor: self model
			getSelected: ('find{1}' format: {tag capitalized}) asSymbol
			setSelected: ('find{1}:' format: {tag capitalized}) asSymbol
			label: tag asString
			help: ('Find code tagged as {1}' format: {tag}))].
	
	^ self newRow: {
		(self newLabel: 'Tag') hResizing: #spaceFill; vResizing: #spaceFill.
		chkCol}! !

!XpfCtTagSearchMorph methodsFor: 'layout' stamp: 'mt 5/5/2009 19:25'!
timeRow

	^ self newRow: {
		(self newLabel: 'Created Time') hResizing: #spaceFill.
		(self
			newDropListFor: self model
			list: #possibleTimes
			getSelected: #selectedTimeIndex
			setSelected: #selectedTimeIndex:
			help: 'Search for tags created at the specific time')
			hResizing: #rigid;
			vResizing: #rigid;
			extent: self class defaultControlWidth @ 24}! !

!XpfCtTagSearchMorph methodsFor: 'appearance' stamp: 'cs 5/4/2009 11:55'!
title

	^ self class title! !

TextAttribute subclass: #XpfComplexAttribute
	instanceVariableNames: 'tag chat chatMorph deleteMorph resizeMorph morph creator timestamp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Core'!

XpfComplexAttribute class
	instanceVariableNames: 'colorTable'!

XpfComplexAttribute class
	instanceVariableNames: 'colorTable'!

!XpfComplexAttribute class methodsFor: 'color mapping' stamp: 'mt 5/5/2009 15:00'!
colorForTag: aSymbol

	colorTable ifNil: [colorTable := Dictionary newFrom: {
		#bad -> (Color red alpha: 0.4).
		#talk -> (Color yellow alpha: 0.4)}].
	
	^ colorTable at: aSymbol ifAbsent: [Color transparent]! !

!XpfComplexAttribute class methodsFor: 'instance creation' stamp: 'mt 5/2/2009 16:11'!
forTag: aSymbol

	^ self new
		tag: aSymbol;
		timestamp: TimeStamp now;
		creator: Utilities authorInitials! !

!XpfComplexAttribute class methodsFor: 'caching' stamp: 'mt 5/1/2009 14:30'!
reset
	"Free all cached values."
	
	colorTable := nil.! !

!XpfComplexAttribute class methodsFor: 'private' stamp: 'mt 5/3/2009 21:19'!
scanChatFrom: strm for: attribute

	| chatCopy |
	chatCopy := Json readFrom: (ByteArray fromHex: (strm upTo: $-)) asString readStream.
	chatCopy do: [:line |
		line at: #timestamp put: (TimeStamp fromString: (line at: #timestamp))].
	attribute chat: chatCopy asOrderedCollection.! !

!XpfComplexAttribute class methodsFor: 'private' stamp: 'mt 5/3/2009 21:19'!
scanCreatorFrom: strm for: attribute

	attribute creator: (ByteArray fromHex: (strm upTo: $-)) asString.! !

!XpfComplexAttribute class methodsFor: 'instance creation' stamp: 'mt 5/3/2009 09:36'!
scanFrom: strm

	| instance substream content |
	instance := self new.

	content := strm upTo: $#.
	(content findTokens: $-) size = 4
		ifFalse: [^ instance. "Parsing failed."].
		
	substream := content readStream.
	self
		scanTimestampFrom: substream for: instance;
		scanTagFrom: substream for: instance;
		scanTagFrom: substream for: instance;
		scanChatFrom: substream for: instance.
	^ instance! !

!XpfComplexAttribute class methodsFor: 'private' stamp: 'mt 5/1/2009 14:37'!
scanTagFrom: strm for: attribute

	attribute tag: (strm upTo: $-) asSymbol.! !

!XpfComplexAttribute class methodsFor: 'private' stamp: 'mt 5/3/2009 21:19'!
scanTimestampFrom: strm for: attribute

	attribute timestamp: (TimeStamp fromString:
		(ByteArray fromHex: (strm upTo: $-)) asString).! !

!XpfComplexAttribute class methodsFor: 'caching' stamp: 'mt 5/5/2009 18:58'!
tags
	"Return a list of all available tags."

	^ #(bad talk)! !

!XpfComplexAttribute methodsFor: 'testing' stamp: 'mt 4/30/2009 16:08'!
= other

	^ self == other! !

!XpfComplexAttribute methodsFor: 'chat' stamp: 'mt 4/30/2009 17:06'!
addChatLine: aString

	self chat add: (Dictionary newFrom: {
		#author -> Utilities authorInitials.
		#timestamp -> TimeStamp now.
		#message -> aString}).
	self changed: #chatLineAdded.! !

!XpfComplexAttribute methodsFor: 'chat' stamp: 'mt 4/30/2009 17:08'!
addDependent: anObject

	(self dependents includes: anObject)
		ifTrue: [^ self].
		
	^ super addDependent: anObject.! !

!XpfComplexAttribute methodsFor: 'accessing' stamp: 'mt 4/29/2009 21:51'!
chat

	chat ifNil: [chat := OrderedCollection new].
	^ chat! !

!XpfComplexAttribute methodsFor: 'accessing' stamp: 'mt 4/30/2009 11:15'!
chat: aCollection

	chat := aCollection.! !

!XpfComplexAttribute methodsFor: 'accessing read-only' stamp: 'mt 5/1/2009 17:11'!
chatMorph

	chatMorph ifNil: [chatMorph := XpfCtChatIconMorph new attribute: self].
	^ chatMorph! !

!XpfComplexAttribute methodsFor: 'html' stamp: 'mt 5/1/2009 15:05'!
closeHtmlOn: aStream 
	"put on the given stream the tag to close the html  
	representation of the receiver"
	aStream nextPutAll: '</span>'! !

!XpfComplexAttribute methodsFor: 'accessing read-only' stamp: 'mt 5/1/2009 14:31'!
color

	^ self class colorForTag: self tag! !

!XpfComplexAttribute methodsFor: 'accessing' stamp: 'mt 5/2/2009 15:47'!
creator

	creator ifNil: [creator := Utilities authorInitials].
	^ creator! !

!XpfComplexAttribute methodsFor: 'accessing' stamp: 'mt 5/2/2009 15:44'!
creator: authorInitials

	creator := authorInitials.! !

!XpfComplexAttribute methodsFor: 'accessing read-only' stamp: 'mt 5/1/2009 17:24'!
deleteMorph

	deleteMorph ifNil: [deleteMorph := XpfCtDeleteIconMorph new attribute: self].
	^ deleteMorph! !

!XpfComplexAttribute methodsFor: 'displaying' stamp: 'mt 5/1/2009 15:37'!
displayOn: aCanvas in: drawBounds
	"Show some stuff on the canvas for this attribute."
	
	aCanvas
		fillRectangle: drawBounds
		color: self color.! !

!XpfComplexAttribute methodsFor: 'accessing' stamp: 'cs 5/4/2009 02:27'!
includesChatLine: aString

	self chat do: [:dict |
		((dict at: #message) includesSubString: aString)
			ifTrue: [^ true]].
	^ false! !

!XpfComplexAttribute methodsFor: 'accessing read-only' stamp: 'mt 5/3/2009 19:21'!
morph

	morph ifNil: [morph := XpfCtRowMorph for: {
		self chatMorph.
		self resizeMorph.
		self deleteMorph}].
	^ morph! !

!XpfComplexAttribute methodsFor: 'html' stamp: 'mt 5/1/2009 15:05'!
openHtmlOn: aStream 
	"put on the given stream the tag to open the html  
	representation of the receiver"
	aStream nextPutAll: '<span>'! !

!XpfComplexAttribute methodsFor: 'event handling' stamp: 'mt 5/1/2009 17:33'!
requestDeletion

	self changed: #deleteRequested with: self.! !

!XpfComplexAttribute methodsFor: 'event handling' stamp: 'mt 5/3/2009 19:47'!
requestResizing
	"No need to give us away because the pure selection in a paragraph will handle this."

	self changed: #resizeRequested.! !

!XpfComplexAttribute methodsFor: 'accessing' stamp: 'mt 5/18/2009 10:40'!
resetMorphs

	morph := nil.
	chatMorph := nil.
	resizeMorph := nil.
	deleteMorph := nil.! !

!XpfComplexAttribute methodsFor: 'accessing read-only' stamp: 'mt 5/3/2009 19:20'!
resizeMorph

	resizeMorph ifNil: [resizeMorph := XpfCtResizeIconMorph new attribute: self].
	^ resizeMorph! !

!XpfComplexAttribute methodsFor: 'accessing' stamp: 'mt 5/1/2009 14:31'!
tag

	^ tag! !

!XpfComplexAttribute methodsFor: 'accessing' stamp: 'mt 5/18/2009 10:42'!
tag: aSymbol
	"Tag this attribute. Different tags allows to display this attribute in different ways."

	tag := (self class tags includes: aSymbol) ifTrue: [aSymbol] ifFalse: [].
	self resetMorphs.! !

!XpfComplexAttribute methodsFor: 'accessing' stamp: 'mt 5/2/2009 15:47'!
timestamp

	timestamp ifNil: [timestamp := TimeStamp now].
	^ timestamp! !

!XpfComplexAttribute methodsFor: 'accessing' stamp: 'mt 5/2/2009 15:45'!
timestamp: aTimeStamp

	timestamp := aTimeStamp.! !

!XpfComplexAttribute methodsFor: 'scanning' stamp: 'mt 5/2/2009 15:51'!
writeChatOn: strm

	| chatCopy |
	chatCopy := self chat collect: [:line |
		Dictionary newFrom: {
			#timestamp -> (line at: #timestamp) printString.
			#author -> (line at: #author).
			#message -> (line at: #message)}].
	strm nextPutAll: (Json render: chatCopy) toHex.
	strm nextPut: $-.! !

!XpfComplexAttribute methodsFor: 'scanning' stamp: 'mt 5/2/2009 15:52'!
writeCreatorOn: strm

	strm nextPutAll: self creator toHex.
	strm nextPut: $-.! !

!XpfComplexAttribute methodsFor: 'scanning' stamp: 'mt 5/2/2009 15:58'!
writeScanOn: strm

	strm nextPut: $h.
	self
		writeTimestampOn: strm;
		writeCreatorOn: strm;
		writeTagOn: strm;
		writeChatOn: strm.
	strm nextPut: $#. "mark end of attribute stream"! !

!XpfComplexAttribute methodsFor: 'scanning' stamp: 'mt 5/1/2009 14:34'!
writeTagOn: strm

	strm nextPutAll: (self tag ifNil: [$0]).
	strm nextPut: $-.! !

!XpfComplexAttribute methodsFor: 'scanning' stamp: 'mt 5/2/2009 16:08'!
writeTimestampOn: strm

	strm nextPutAll: self timestamp printString toHex.
	strm nextPut: $-.! !

!String methodsFor: '*xpf-codetalk' stamp: 'mt 5/2/2009 15:52'!
toHex

	^ self
		inject: ''
		into: [:result :char | result, char hex]! !

!OBCmdMultiView methodsFor: '*xpf-codetalk-override' stamp: 'mt 4/28/2009 08:46'!
execute
	|viewType|
	viewType := OBChoiceRequest labels: #(Source Bytecode Decompile PrettyPrint Xpf).
	self styler: SHTextStylerST80 new.
	viewType ifNotNil: [self perform: (#toggle, viewType) asSymbol]! !

!OBCmdMultiView methodsFor: '*xpf-codetalk' stamp: 'cs 1/15/2009 17:24'!
styler: styler
	| panel morph |
	
	panel := requestor browser panels 
		detect: [ :each | each isKindOf: OBDefinitionPanel ]
		ifNone: [ ^ self ].
	morph := OBPluggableTextMorph allSubInstances
		detect: [ :each | each model == panel ]
		ifNone: [ ^ self ].
	morph styler: (styler
		view: morph;
		yourself).! !

!OBCmdMultiView methodsFor: '*xpf-codetalk' stamp: 'topa 2/13/2009 13:47'!
toggleXpf

	self styler: XpfTextStyler new.
	self setTextInDefinitionPanel: target source.! !

OBLazyListMorph subclass: #XpfLazyListMorph
	instanceVariableNames: 'showCategories'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Core'!

!XpfLazyListMorph methodsFor: 'displaying' stamp: 'mt 5/5/2009 14:48'!
display: item  atRow: row on: canvas
	"display the given item at row row"
	| drawBounds top icon node |
	drawBounds := self drawBoundsForRow: row.
	drawBounds := drawBounds intersect: self bounds.
	node := listSource model children at: row.
	
	(listSource backgroundColorAt: row) ifNotNilDo: [:aColor | canvas fillRectangle: drawBounds color: aColor].

	"(node isKindOf: OBClassNode)
		ifTrue: [
			self indicateMarkupForClass: node on: canvas in: drawBounds.
			drawBounds := drawBounds left: drawBounds left + 2 + 6]."
	
	(listSource iconAt: row) ifNotNilDo: 
		[:name |
		icon := OBMorphicIcons iconNamed: name.
		top := drawBounds top + (drawBounds height - icon height // 2).
		canvas translucentImage: icon at: drawBounds left @ top.
		drawBounds := drawBounds left: drawBounds left + icon width + 2].
		
	item isText
		ifTrue: [canvas
			drawString: item
			in: drawBounds
			font: (font emphasized: (item emphasisAt: 1))
			color: (self colorForRow: row).
			drawBounds := drawBounds left: drawBounds left + ((font emphasized: (item emphasisAt: 1)) widthOfString: item)]
		ifFalse: [canvas
			drawString: item
			in: drawBounds
			font: font
			color: (self colorForRow: row).
			drawBounds := drawBounds left: drawBounds left + (font widthOfString: item)].

	(node isKindOf: OBMethodNode) "& (row ~= selectedRow)"
		ifTrue: [drawBounds := self
			displayCategoryFor: node
			in: drawBounds
			on: canvas].
		
	(node isKindOf: OBMethodNode)
		ifTrue: [
			drawBounds := self
				indicateMarkupFor: node
				on: canvas
				in: drawBounds].! !

!XpfLazyListMorph methodsFor: 'displaying' stamp: 'mt 5/5/2009 14:59'!
displayCategoryFor: aMethodNode in: drawBounds on: aCanvas

	| txt |
	self updateProperties.
	aMethodNode theClass == nil ifTrue: [^ drawBounds].
	
	txt := ' ({1})' format: {aMethodNode categoryText}.
	showCategories ifTrue: [aCanvas
		drawString: txt
		in: drawBounds
		font: font
		color: ((aMethodNode categoryText = 'as yet unclassified')
			ifTrue: [Color red]
			ifFalse: [Color gray])].
		
	^ drawBounds left: drawBounds left + (font widthOfString: txt asString) + 2! !

!XpfLazyListMorph methodsFor: 'displaying' stamp: 'mt 5/5/2009 14:59'!
indicateMarkupFor: aMethodNode on: aCanvas in: drawBounds

	| icon top newBounds |
	aMethodNode theClass ifNil: [^ drawBounds].
	
	newBounds := drawBounds.
	(aMethodNode sourceText asText attributesOfKind: XpfComplexAttribute)
		do: [ :attr |
			icon := attr tag = #bad
				ifTrue: [MenuIcons xpfCtSmallMarkBadIcon]
				ifFalse: [MenuIcons xpfCtSmallMarkCommentIcon].
			top := newBounds top + (newBounds height - icon height // 2) + 1.
			aCanvas translucentImage: icon at: newBounds left @ top.
			newBounds := newBounds left: newBounds left + icon width].
	^ newBounds! !

!XpfLazyListMorph methodsFor: 'displaying' stamp: 'mt 5/5/2009 14:56'!
indicateMarkupForClass: aClassNode on: aCanvas in: drawBounds

	| left top icon |
	(aClassNode methods anySatisfy: [:node | node source asText includesAttributesOfKind: XpfComplexAttribute])
		ifTrue: [
			icon := MenuIcons markGoodIcon.
			top := drawBounds top + (drawBounds height - icon height // 2).
			left := drawBounds left - 5.
			aCanvas
				translucentImage: icon
				at: left @ top].! !

!XpfLazyListMorph methodsFor: 'accessing' stamp: 'mt 5/1/2009 17:03'!
updateProperties

	showCategories ifNil: [showCategories := 
		(Preferences
			preferenceAt: #showProtocolForMessage
			ifAbsent: [Preference new rawValue: false])
			preferenceValue].! !

!Text methodsFor: '*xpf-codetalk' stamp: 'mt 5/1/2009 17:00'!
attributesOfKind: anAttrClass
	"Return a collection with all instances of the given attribute class that can be found in this text."
	
	| result |
	result := OrderedCollection new.
	runs values do: [:array |
		array do: [:attr | (attr isKindOf: anAttrClass)
			ifTrue: [result addIfNotPresent: attr]]].
	^ result! !

!Text methodsFor: '*xpf-codetalk' stamp: 'mt 5/18/2009 11:16'!
attributesOfKind: anAttrClass from: start to: stop
	"Return a collection with all instances of the given attribute class that can be found in the specified range."
	
	| part result |
	start > stop ifTrue: [
		^ (self attributesAt: start) select: [:attr | attr isKindOf: anAttrClass]].
	
	result := OrderedCollection new.
	part := self copyFrom: start to: stop.
	part runs withStartStopAndValueDo: [:i :j :attributes |
		attributes
			select: [:attr | attr isKindOf: anAttrClass]
			thenDo: [:attr | result addIfNotPresent: attr]].
	^ result.! !

!Text methodsFor: '*xpf-codetalk' stamp: 'mt 5/1/2009 14:01'!
includesAttributesOfKind: anAttrClass
	"Check whether this text has at least one run that includes an attribute of the given class."
	
	runs values do: [:array |
		array do: [:attr | (attr isKindOf: anAttrClass) ifTrue: [^ true]]].
	^ false! !

!Text methodsFor: '*xpf-codetalk' stamp: 'mt 5/3/2009 18:37'!
includesAttributesOfKind: anAttrClass from: start to: stop
	"Check whether this text has at least one run that includes an attribute of the given class in the specified range."
	
	| part |
	part := self copyFrom: start to: stop.
	part runs values do: [:array |
		array do: [:attr | (attr isKindOf: anAttrClass) ifTrue: [^ true]]].
	^ false! !

!Text methodsFor: '*xpf-codetalk' stamp: 'mt 5/1/2009 17:08'!
removeAttribute: attr
	
	self removeAttribute: attr from: 1 to: self size.! !

!Text methodsFor: '*xpf-codetalk' stamp: 'mt 5/1/2009 13:56'!
removeAttribute: anAttribute around: index
	
	self removeAttribute: anAttribute around: index and: index.! !

!Text methodsFor: '*xpf-codetalk' stamp: 'mt 5/1/2009 15:08'!
removeAttribute: anAttribute around: leftIndex and: rightIndex
	"Remove the given attribute from all indices that are adjacent to the given ones."

	| start stop |
	start := leftIndex.
	[start > 0 and: [(self attributesAt: start) includes: anAttribute]]
		whileTrue: [start := start - 1].
	stop := rightIndex.
	[stop <= self size and: [(self attributesAt: stop) includes: anAttribute]]
		whileTrue: [stop := stop + 1].

	self removeAttribute: anAttribute from: (start max: 0) to: (stop min: runs size).! !

!Text methodsFor: '*xpf-codetalk' stamp: 'mt 5/1/2009 17:08'!
removeAttribute: attr from: start to: stop
	"Removes an attribute kind over the interval start to stop."
	
	runs := runs
		copyReplaceFrom: start
		to: stop
		with: ((runs copyFrom: start to: stop)	mapValues:
			[:attributes | attributes reject: [:each | each = attr]])! !

Object subclass: #XpfCodeTalk
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Core'!

!XpfCodeTalk class methodsFor: 'initialization' stamp: 'cs 5/4/2009 11:48'!
initialize
	"Resend this message whenever you checkout a new version of CodeTalk."
	
	super initialize.
	self
		initializeMonticello;
		initializePreferences;
		initializeOtherClasses;
		registerInToolsFlap;
		registerInWorldMenu.! !

!XpfCodeTalk class methodsFor: 'initialization' stamp: 'mt 5/2/2009 09:26'!
initializeMonticello

	(MCMethodDefinition allInstVarNames includes: #sourceText)
		ifFalse:[MCMethodDefinition addInstVarName: #sourceText].! !

!XpfCodeTalk class methodsFor: 'initialization' stamp: 'mt 5/2/2009 09:27'!
initializeOtherClasses
	
	(Smalltalk includesKey: #XpfEditor)
		ifTrue: [XpfEditor initializeCmdKeyShortcuts].
		
	(Smalltalk includesKey: #XpfComplexAttribute)
		ifTrue: [XpfComplexAttribute reset].! !

!XpfCodeTalk class methodsFor: 'initialization' stamp: 'mt 5/2/2009 11:38'!
initializePreferences
		
	(Preferences dictionaryOfPreferences includesKey: #showLineNumbers)
		ifFalse: [Preferences
			addBooleanPreference: #showLineNumbers
			category: #CodeTalk
			default: false
			balloonHelp: 'Enables line numbers in front of each codeline.'].
		
	(Preferences dictionaryOfPreferences includesKey: #showProtocolForMessage)
		ifFalse: [Preferences
			addBooleanPreference: #showProtocolForMessage
			category: #CodeTalk
			default: false
			balloonHelp: 'Displays the protocol name after each message name in the code browser.'].
		
	Preferences setPreference: #confirmFirstUseOfStyle toValue: false.
	Preferences setPreference: #alwaysShowVScrollbar toValue: false.! !

!XpfCodeTalk class methodsFor: 'as yet unclassified' stamp: 'mt 5/9/2009 17:30'!
registerInToolsFlap

	self environment at: #Flaps ifPresent: [ :class |
		class
			registerQuad: {
				#XpfCtTagSearchMorph.
				#prototypicalToolWindow.
				XpfCtTagSearchMorph title.
				'A tool for discovering methods that contains different kind of markup'}
			forFlapNamed: 'Tools';
			replaceToolsFlap.
		class
			registerQuad: {
				#XpfCtTagAnalysisMorph.
				#prototypicalToolWindow.
				XpfCtTagAnalysisMorph title.
				'A tool for analyze markup of methods'}
			forFlapNamed: 'Tools';
			replaceToolsFlap].! !

!XpfCodeTalk class methodsFor: 'as yet unclassified' stamp: 'mt 5/9/2009 17:30'!
registerInWorldMenu

	self environment at: #TheWorldMenu ifPresent: [ :class |
		class registerOpenCommand: {
			XpfCtTagSearchMorph title.
			{XpfCtTagSearchMorph. #open}}.
		class registerOpenCommand: {
			XpfCtTagAnalysisMorph title.
			{XpfCtTagAnalysisMorph. #open}}]! !

!XpfCodeTalk methodsFor: 'testing' stamp: 'mt 5/25/2009 13:57'!
todo

	"Feature chat: here."! !

Object subclass: #XpfCtCodeChatModel
	instanceVariableNames: 'attribute searchString'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Models'!

!XpfCtCodeChatModel methodsFor: 'adding' stamp: 'mt 5/3/2009 14:14'!
addMessage: aMessage

	aMessage ifEmpty: [^ self].
	self attribute ifNil: [^ self].
	self attribute addChatLine: aMessage.
	self
		changed: #messages;
		changed: #newMessage;
		changed: #lastIndex;
		changed: #hasMessages.! !

!XpfCtCodeChatModel methodsFor: 'accessing' stamp: 'mt 4/29/2009 22:22'!
attribute

	^ attribute! !

!XpfCtCodeChatModel methodsFor: 'accessing' stamp: 'mt 4/29/2009 22:22'!
attribute: anXpfComplexAttribute

	attribute := anXpfComplexAttribute.
	self changed: #messages.! !

!XpfCtCodeChatModel methodsFor: 'accessing' stamp: 'mt 5/3/2009 14:14'!
hasMessages

	^ self attribute notNil and: [self attribute chat isEmpty not].! !

!XpfCtCodeChatModel methodsFor: 'accessing' stamp: 'mt 5/1/2009 18:39'!
lastIndex
	"Used to show the latest message"
	
	self attribute ifNil: [^ 0].
	^ self attribute chat size! !

!XpfCtCodeChatModel methodsFor: 'accessing' stamp: 'mt 5/3/2009 14:05'!
messages

	self attribute ifNil: [^ #()].
	
	^ self attribute chat collect: [:line | | msgMorph |
		msgMorph := XpfCtMessageMorph
		timestamp: (line at: #timestamp)
		author: (line at: #author)
		message: (line at: #message).
		self searchString ifNotEmpty: [msgMorph highlightIf: self searchString].
		msgMorph]! !

!XpfCtCodeChatModel methodsFor: 'accessing' stamp: 'mt 5/1/2009 18:07'!
newMessage

	^ ''! !

!XpfCtCodeChatModel methodsFor: 'accessing' stamp: 'mt 5/3/2009 13:58'!
searchString

	searchString ifNil: [searchString := ''].
	^ searchString! !

!XpfCtCodeChatModel methodsFor: 'accessing' stamp: 'mt 5/3/2009 14:21'!
searchString: aString

	searchString := aString.
	self changed: #messages.! !

Object subclass: #XpfCtGraphModel
	instanceVariableNames: 'inspector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Models'!

!XpfCtGraphModel class methodsFor: 'instance creation' stamp: 'mt 5/9/2009 14:18'!
newForInspector: anMczInspector

	^ self new
		inspector: anMczInspector! !

!XpfCtGraphModel methodsFor: 'accessing' stamp: 'mt 5/9/2009 14:19'!
inspector

	^ inspector! !

!XpfCtGraphModel methodsFor: 'accessing' stamp: 'mt 5/9/2009 14:02'!
inspector: anObject

	self inspector ifNotNilDo: [:i | i removeDependent: self].
	inspector := anObject.
	self inspector ifNotNilDo: [:i | i addDependent: self].! !

!XpfCtGraphModel methodsFor: 'processing' stamp: 'mt 5/9/2009 15:56'!
processFinish

	self subclassResponsiblity.! !

!XpfCtGraphModel methodsFor: 'processing' stamp: 'mt 5/9/2009 14:07'!
processResult: aDictionary

	self subclassResponsiblity.! !

!XpfCtGraphModel methodsFor: 'processing' stamp: 'mt 5/9/2009 14:55'!
reset

	self changed: #reset.! !

!XpfCtGraphModel methodsFor: 'updating' stamp: 'mt 5/9/2009 15:56'!
update: aSymbol

	super update: aSymbol.
	aSymbol = #isRunning ifTrue: [
		self inspector isRunning ifTrue: [self reset]].
	aSymbol = #done ifTrue: [self processFinish].	! !

!XpfCtGraphModel methodsFor: 'updating' stamp: 'mt 5/9/2009 14:07'!
update: aSymbol with: aDictionary

	super update: aSymbol with: aDictionary.
	aSymbol = #resultFound ifTrue: [self processResult: aDictionary].! !

XpfCtGraphModel subclass: #XpfCtTestGraphModel
	instanceVariableNames: 'versionCount attributeCount currentVersionName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Models'!

!XpfCtTestGraphModel methodsFor: 'initialize-release' stamp: 'mt 5/9/2009 15:57'!
initialize

	super initialize.
	self resetCache.! !

!XpfCtTestGraphModel methodsFor: 'processing' stamp: 'mt 5/15/2009 14:46'!
pointFound

	versionCount > 0
		ifTrue: [self changed: #newPoint with: versionCount@attributeCount].! !

!XpfCtTestGraphModel methodsFor: 'processing' stamp: 'mt 5/15/2009 14:39'!
processFinish

	self pointFound.! !

!XpfCtTestGraphModel methodsFor: 'processing' stamp: 'mt 5/15/2009 14:39'!
processResult: aDictionary

	currentVersionName ifEmpty: [
		currentVersionName := (aDictionary at: #info) name.
		attributeCount := (aDictionary at: #attributes) size.
		versionCount := ((aDictionary at: #info) name findTokens: $.) last asNumber.
		^ self].

	currentVersionName = (aDictionary at: #info) name
		ifTrue: [
			attributeCount := attributeCount + (aDictionary at: #attributes) size]
		ifFalse: [
			self pointFound.
			versionCount := ((aDictionary at: #info) name findTokens: $.) last asNumber.
			currentVersionName := (aDictionary at: #info) name.
			attributeCount := (aDictionary at: #attributes) size].! !

!XpfCtTestGraphModel methodsFor: 'processing' stamp: 'mt 5/9/2009 15:57'!
reset

	super reset.
	self resetCache.! !

!XpfCtTestGraphModel methodsFor: 'processing' stamp: 'mt 5/9/2009 15:57'!
resetCache

	versionCount := 0.
	attributeCount := 0.
	currentVersionName := ''.! !

Object subclass: #XpfCtMczCache
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Models'!

!XpfCtMczCache class methodsFor: 'private' stamp: 'mt 5/15/2009 14:28'!
addMczToCache: aStream at: aMczFileName
	"Need to write a clean zip."
	
	^ ZipArchive new
		readFrom: aStream;
		writeToFileNamed: (self cacheDirectory fullNameFor: aMczFileName).! !

!XpfCtMczCache class methodsFor: 'private' stamp: 'mt 5/15/2009 14:35'!
cacheAt: aFileName

	^ ZipArchive new
		readFrom: (self cacheDirectory readOnlyFileNamed: aFileName)! !

!XpfCtMczCache class methodsFor: 'caching' stamp: 'mt 5/15/2009 14:34'!
cacheAt: aFileName ifNoneGetFrom: anHttpRepository
	"Return a zip archive for the mcz file."

	^ (self includes: aFileName)
		ifTrue: [self cacheAt: aFileName]
		ifFalse: [anHttpRepository
				readStreamForFileNamed: aFileName
				do: [:strm | self addMczToCache: strm at: aFileName]]! !

!XpfCtMczCache class methodsFor: 'private' stamp: 'mt 5/15/2009 12:45'!
cacheDirectory

	^ (FileDirectory default directoryNamed: 'codetalk-cache')
		assureExistence;
		yourself! !

!XpfCtMczCache class methodsFor: 'private' stamp: 'mt 5/15/2009 13:25'!
includes: aFileName

	^ self cacheDirectory includesKey: aFileName! !

Object subclass: #XpfCtMczInspector
	instanceVariableNames: 'repositories isRunning process zip currentRepository currentMethodDefinition followAncestors currentAncestors classFilter selectorFilter currentVersionInfo lastDefinition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Models'!

XpfCtMczInspector class
	instanceVariableNames: 'exampleCache'!

XpfCtMczInspector class
	instanceVariableNames: 'exampleCache'!

!XpfCtMczInspector class methodsFor: 'examples' stamp: 'mt 5/19/2009 13:31'!
exampleVersions

	| instance result repo |
	exampleCache ifNotNilDo: [:c | ^ c].
	
	result := OrderedCollection new.
	instance := self newForPackageNamed: 'Xpf-CodeTalk'.
	repo := instance repositories first key.
	(repo readableFileNames
		select: [:filename | filename startsWith: 'Xpf-CodeTalk'])
		do: [:filename |
			instance zip: (XpfCtMczCache
				cacheAt: filename
				ifNoneGetFrom: repo).
			instance loadVersionInfo.
			
			(instance zip memberNamed: 'snapshot.bin') ifNotNilDo: [:bin |
			[(DataStream on: bin contentStream) next definitions
				select: [:def | def isMethodDefinition and: [def className = 'XpfCodeTalk' and: [def selector = #todo]]]
				thenDo: [:def | result add: (XpfCtRemoteMethodVersion
					repository: repo
					info: instance info
					definition: def)]]
				on: Error
				do: [:fallThrough | "Unknown content in snapshot.bin!!"]]] displayingProgress: 'Preparing...'.
	exampleCache := result.
	^ result! !

!XpfCtMczInspector class methodsFor: 'instance creation' stamp: 'mt 5/19/2009 11:52'!
newForPackageNamed: aString

	| instance pkg |
	instance := self new.
	pkg := MCPackage named: aString.
	pkg hasWorkingCopy ifTrue: [pkg workingCopy repositoryGroup repositories
		select: [:r | r isKindOf: MCHttpRepository]
		thenDo: [:r | instance addRepository: r include: {aString}]].
	^ instance! !

!XpfCtMczInspector methodsFor: 'adding repos' stamp: 'mt 5/9/2009 12:22'!
addRepository: anHttpRepository include: somePackageNames
	"Add the given repo and decide which packages to use when looking at the contents of the repo later."
	
	self repositories add: (anHttpRepository -> somePackageNames).! !

!XpfCtMczInspector methodsFor: 'mcz reading' stamp: 'mt 5/9/2009 12:56'!
associate: tokens

	| result |
	result := Dictionary new.
	tokens pairsDo: [:key :value | 
					value isString ifFalse: [value := value collect: [:ea | self associate: ea]].
					result at: key put: value].
	^ result! !

!XpfCtMczInspector methodsFor: 'mcz reading' stamp: 'mt 5/13/2009 15:07'!
extractInfoFrom: dict

	^ MCVersionInfo
		name: (dict at: #name ifAbsent: [''])
		id: (UUID fromString: (dict at: #id))
		message: (dict at: #message ifAbsent: [''])
		date: ([Date fromString: (dict at: #date) ] on: Error do: [ :ex | ex return: nil ])
		time: ([ Time fromString:(dict at: #time)] on: Error do: [ :ex | ex return: nil ])
		author: (dict at: #author ifAbsent: [''])
		ancestors: ((dict at: #ancestors ifAbsent: [#()]) collect: [:ea | self extractInfoFrom: ea])
		stepChildren: ((dict at: #stepChildren ifAbsent: [#()]) collect: [:ea | self extractInfoFrom: ea])! !

!XpfCtMczInspector methodsFor: 'accessing' stamp: 'mt 5/13/2009 14:14'!
followAncestors

	followAncestors ifNil: [self followAncestors: false].
	^ followAncestors! !

!XpfCtMczInspector methodsFor: 'accessing' stamp: 'mt 5/13/2009 14:13'!
followAncestors: anObject

	followAncestors := anObject.! !

!XpfCtMczInspector methodsFor: 'debugging' stamp: 'mt 5/19/2009 11:58'!
info

	^ currentVersionInfo! !

!XpfCtMczInspector methodsFor: 'inspecting' stamp: 'mt 5/25/2009 14:55'!
inspectDefinitions
	"Read all definitions from 'snapshot.bin' in current zip."
	
	(zip memberNamed: 'snapshot.bin') ifNotNilDo: [:bin |
		[self	inspectDefinitions:
			(DataStream on: bin contentStream) next definitions]
		on: Error
		do: [:fallThrough | Transcript show: 'Unknown content in snapshot.bin!!'; cr]].! !

!XpfCtMczInspector methodsFor: 'inspecting' stamp: 'mt 5/25/2009 15:32'!
inspectDefinitions: someMCDefinitions

	someMCDefinitions
		select: [:def | def isMethodDefinition]
		thenDo: [:methodDef |
			(classFilter = methodDef actualClass and: [selectorFilter = methodDef selector])
				ifTrue: [
					lastDefinition = methodDef ifFalse:[ 
						self changed: #versionFound with: (Dictionary newFrom: {
						#repository -> currentRepository.
						#definition -> methodDef.
						#info -> currentVersionInfo})].
					lastDefinition := methodDef].
			
			currentMethodDefinition := methodDef.
			self inspectSourceText:
				currentMethodDefinition sourceText].! !

!XpfCtMczInspector methodsFor: 'inspecting' stamp: 'mt 5/15/2009 14:37'!
inspectRepository: anHttpRepository include: somePackageNames

	currentRepository := anHttpRepository.
	(self followAncestors
		ifTrue: [currentRepository readableFileNames]
		ifFalse: [currentRepository readableFileNames reversed])
			select: [:filename | somePackageNames anySatisfy: [:name | filename startsWith: name]]
			thenDo: [:filename |
					zip := XpfCtMczCache cacheAt: filename ifNoneGetFrom: currentRepository.
					self loadVersionInfo.
					self followAncestors
						ifFalse: [self inspectDefinitions]
						ifTrue: [(currentAncestors isNil or: [currentAncestors anySatisfy: [:anc | anc name = currentVersionInfo name]])
							ifTrue: [self inspectDefinitions].
							currentAncestors := currentVersionInfo ancestors]].! !

!XpfCtMczInspector methodsFor: 'inspecting' stamp: 'mt 5/9/2009 17:56'!
inspectSourceText: aText

	| result |
	result := aText attributesOfKind: XpfComplexAttribute.

	self changed: #resultFound with: (Dictionary newFrom: {
		#repository -> currentRepository.
		#definition -> currentMethodDefinition.
		#info -> currentVersionInfo.
		#attributes -> result}).! !

!XpfCtMczInspector methodsFor: 'inspecting' stamp: 'mt 5/25/2009 14:01'!
inspectVersionsFor: aSelector in: aClassName

	classFilter := aClassName.
	selectorFilter := aSelector.
	self followAncestors: true.
	self start.! !

!XpfCtMczInspector methodsFor: 'accessing' stamp: 'mt 5/9/2009 12:03'!
isRunning

	isRunning ifNil: [self isRunning: false].
	^ isRunning! !

!XpfCtMczInspector methodsFor: 'accessing' stamp: 'mt 5/9/2009 12:03'!
isRunning: anObject

	isRunning := anObject.
	self changed: #isRunning.! !

!XpfCtMczInspector methodsFor: 'mcz reading' stamp: 'mt 5/9/2009 13:02'!
loadVersionInfo

	currentVersionInfo := self extractInfoFrom: (self parseMember: 'version')! !

!XpfCtMczInspector methodsFor: 'mcz reading' stamp: 'mt 5/9/2009 12:56'!
parseMember: fileName

	| tokens |
	tokens := (self scanner scanTokens: (zip contentsOf: fileName)) first.
	^ self associate: tokens! !

!XpfCtMczInspector methodsFor: 'debugging' stamp: 'mt 5/9/2009 14:38'!
print

	Transcript show: ('Next Result: {1}>>{2} ({3}) - {4}' format: {
		currentMethodDefinition className.
		currentMethodDefinition selector.
		currentMethodDefinition category.
		currentVersionInfo name}); cr.! !

!XpfCtMczInspector methodsFor: 'accessing' stamp: 'mt 5/9/2009 12:01'!
repositories

	repositories ifNil: [self repositories: OrderedCollection new].
	^ repositories! !

!XpfCtMczInspector methodsFor: 'accessing' stamp: 'mt 5/9/2009 12:01'!
repositories: anObject

	repositories := anObject.! !

!XpfCtMczInspector methodsFor: 'mcz reading' stamp: 'mt 5/9/2009 12:56'!
scanner

	^ MCScanner! !

!XpfCtMczInspector methodsFor: 'inspecting' stamp: 'mt 5/9/2009 15:55'!
start

	process ifNotNil: [self stop].
	process := [
		self isRunning: true.
		self repositories do: [:repo |
			self
				inspectRepository: repo key
				include: repo value].
		self
			isRunning: false;
			changed: #done] fork priority: 30.! !

!XpfCtMczInspector methodsFor: 'inspecting' stamp: 'mt 5/9/2009 12:13'!
stop

	process ifNotNilDo: [:p | p terminate].
	process := nil.
	self isRunning: false.! !

!XpfCtMczInspector methodsFor: 'accessing' stamp: 'mt 5/19/2009 11:56'!
zip

	^ zip! !

!XpfCtMczInspector methodsFor: 'accessing' stamp: 'mt 5/19/2009 11:56'!
zip: anObject

	zip := anObject.! !

Object subclass: #XpfCtRemoteMethodVersion
	instanceVariableNames: 'repository definition info'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Versions'!

!XpfCtRemoteMethodVersion class methodsFor: 'as yet unclassified' stamp: 'mt 5/19/2009 11:38'!
repository: aRepository info: aVersionInfo definition: aMethodDefinition

	^ self new
		repository: aRepository;
		info: aVersionInfo;
		definition: aMethodDefinition! !

!XpfCtRemoteMethodVersion methodsFor: 'accessing' stamp: 'mt 5/19/2009 16:22'!
= other

	^ self species = other species and: [self info timeStamp = other info timeStamp].! !

!XpfCtRemoteMethodVersion methodsFor: 'accessing' stamp: 'mt 5/19/2009 11:28'!
category

	^ self definition category! !

!XpfCtRemoteMethodVersion methodsFor: 'accessing' stamp: 'mt 5/19/2009 11:18'!
definition
	"This should be a Monticello method definition."

	^ definition! !

!XpfCtRemoteMethodVersion methodsFor: 'accessing' stamp: 'mt 5/19/2009 11:17'!
definition: anObject

	definition := anObject.! !

!XpfCtRemoteMethodVersion methodsFor: 'accessing' stamp: 'mt 5/19/2009 16:07'!
fileIn
	(self theClass) ifNotNilDo: [:class | class
						 				compile: self source 
										classified: self category 
										withStamp: self stamp 
										notifying: nil]! !

!XpfCtRemoteMethodVersion methodsFor: 'accessing' stamp: 'mt 5/19/2009 11:18'!
info
	"This should be a Monticello version information object."

	^ info! !

!XpfCtRemoteMethodVersion methodsFor: 'accessing' stamp: 'mt 5/19/2009 11:17'!
info: anObject

	info := anObject.! !

!XpfCtRemoteMethodVersion methodsFor: 'accessing' stamp: 'mt 5/19/2009 11:17'!
repository

	^ repository! !

!XpfCtRemoteMethodVersion methodsFor: 'accessing' stamp: 'mt 5/19/2009 11:17'!
repository: anObject

	repository := anObject.! !

!XpfCtRemoteMethodVersion methodsFor: 'accessing' stamp: 'mt 5/19/2009 11:24'!
selector

	^ self definition selector! !

!XpfCtRemoteMethodVersion methodsFor: 'accessing' stamp: 'mt 5/19/2009 15:36'!
source

	^ self sourceText! !

!XpfCtRemoteMethodVersion methodsFor: 'accessing' stamp: 'mt 5/19/2009 11:26'!
sourceText

	^ self definition sourceText! !

!XpfCtRemoteMethodVersion methodsFor: 'accessing' stamp: 'mt 5/25/2009 14:06'!
stamp

	^ self definition timeStamp! !

!XpfCtRemoteMethodVersion methodsFor: 'accessing' stamp: 'mt 5/19/2009 11:24'!
theClass

	^ self definition actualClass! !

!XpfCtRemoteMethodVersion methodsFor: 'accessing' stamp: 'mt 5/19/2009 15:38'!
theClassName

	^ self definition className! !

Object subclass: #XpfCtTagAnalysisModel
	instanceVariableNames: 'inspector repositoryName username password filter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Models'!

!XpfCtTagAnalysisModel methodsFor: 'testing' stamp: 'mt 5/9/2009 17:24'!
canAnalyze

	^ self repositoryName notEmpty! !

!XpfCtTagAnalysisModel methodsFor: 'accessing' stamp: 'mt 5/9/2009 17:36'!
filter

	filter ifNil: [self filter: 'Xpf-CodeTalk'].
	^ filter! !

!XpfCtTagAnalysisModel methodsFor: 'accessing' stamp: 'mt 5/9/2009 17:38'!
filter: anObject

	filter := anObject.
	self changed: #filter.! !

!XpfCtTagAnalysisModel methodsFor: 'accessing' stamp: 'mt 5/13/2009 15:51'!
inspector

	inspector ifNil: [self inspector: XpfCtMczInspector new].
	^ inspector! !

!XpfCtTagAnalysisModel methodsFor: 'accessing' stamp: 'mt 5/9/2009 17:39'!
inspector: anObject

	inspector ifNotNilDo: [:i | i removeDependent: self].
	inspector := anObject.
	inspector ifNotNilDo: [:i | i addDependent: self].
	self changed: #inspector.! !

!XpfCtTagAnalysisModel methodsFor: 'testing' stamp: 'mt 5/9/2009 17:53'!
isNotAnalyzing

	^ self inspector isRunning not! !

!XpfCtTagAnalysisModel methodsFor: 'accessing' stamp: 'mt 5/9/2009 17:27'!
label

	^ self inspector isRunning
		ifTrue: ['Stop Analysis']
		ifFalse: ['Start Analysis'].! !

!XpfCtTagAnalysisModel methodsFor: 'accessing' stamp: 'mt 5/9/2009 17:35'!
password

	password ifNil: [self password: 'gsgs'].
	^ password! !

!XpfCtTagAnalysisModel methodsFor: 'accessing' stamp: 'mt 5/9/2009 17:12'!
password: anObject

	password := anObject.
	self changed: #password.! !

!XpfCtTagAnalysisModel methodsFor: 'accessing' stamp: 'mt 5/9/2009 17:35'!
repositoryName

	repositoryName ifNil: [self repositoryName: 'xpforums'].
	^ repositoryName! !

!XpfCtTagAnalysisModel methodsFor: 'accessing' stamp: 'mt 5/9/2009 17:52'!
repositoryName: anObject

	repositoryName := anObject.
	self
		changed: #repositoryName;
		changed: #canAnalyze.! !

!XpfCtTagAnalysisModel methodsFor: 'analyzing' stamp: 'mt 5/9/2009 17:41'!
startStop

	self inspector isRunning
		ifTrue: [
			self inspector stop.
			self inspector repositories: OrderedCollection new]
		ifFalse: [
			self inspector
				addRepository: (MCHttpRepository
					location: 'http://www.hpi.uni-potsdam.de/hirschfeld/squeaksource/', self repositoryName
					user: self username
					password: self password)
				include: {self filter}.
			self inspector start].! !

!XpfCtTagAnalysisModel methodsFor: 'updating' stamp: 'mt 5/9/2009 17:51'!
update: aSymbol

	super update: aSymbol.
	aSymbol = #isRunning ifTrue: [
		self
			changed: #label;
			changed: #isAnalyzing].! !

!XpfCtTagAnalysisModel methodsFor: 'accessing' stamp: 'mt 5/9/2009 17:35'!
username

	username ifNil: [self username: 'gs'].
	^ username! !

!XpfCtTagAnalysisModel methodsFor: 'accessing' stamp: 'mt 5/9/2009 17:12'!
username: anObject

	username := anObject.
	self changed: #username.! !

Object subclass: #XpfCtTagSearchHelper
	instanceVariableNames: 'results classMatchers methodMatchers attributeMatchers typeMatchers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Models'!

!XpfCtTagSearchHelper methodsFor: 'matching' stamp: 'cs 5/2/2009 16:43'!
addAttributeMatcher: matcher

	self attributeMatchers add: matcher.! !

!XpfCtTagSearchHelper methodsFor: 'matching' stamp: 'cs 5/2/2009 16:43'!
addMethodMatcher: matcher

	self methodMatchers add: matcher.! !

!XpfCtTagSearchHelper methodsFor: 'searching' stamp: 'mt 5/5/2009 20:09'!
addToResultClass: mrClass symbol: mrSel

	| result |
	result := MethodReference new
		setStandardClass: mrClass
		methodSymbol: mrSel.
	self pvtResults add: result.
	self changed: #resultFound with: result.! !

!XpfCtTagSearchHelper methodsFor: 'matching' stamp: 'cs 5/3/2009 17:15'!
addTypeMatcher: matcher

	self typeMatchers add: matcher.! !

!XpfCtTagSearchHelper methodsFor: 'accessing' stamp: 'cs 5/2/2009 15:31'!
attributeMatchers

	^ attributeMatchers! !

!XpfCtTagSearchHelper methodsFor: 'accessing' stamp: 'cs 5/2/2009 15:31'!
attributeMatchers: anObject

	attributeMatchers := anObject.! !

!XpfCtTagSearchHelper methodsFor: 'updating' stamp: 'mt 5/7/2009 08:56'!
doWithProgress: aCollection block: aDoBlock message: aString
	"This method is used to signal asynchronous progress from a non-gui thread."
	
	| startTime |
	startTime := TimeStamp now.
	aCollection
		withIndexDo: [:e :i| 
			(TimeStamp now - startTime) seconds >= 1
				ifTrue: [
					self changed: #progressChanged with: {aString. i. aCollection size}. 
					startTime := TimeStamp now].
			aDoBlock value: e].
	self changed: #progressChanged with: {''. aCollection size. aCollection size}.! !

!XpfCtTagSearchHelper methodsFor: 'initialization' stamp: 'cs 5/3/2009 17:16'!
initialize

	self
		methodMatchers: OrderedCollection new;
		typeMatchers: OrderedCollection new;
		attributeMatchers: OrderedCollection new;
		results: Set new;
		addMethodMatcher: [:sel | sel isDoIt not]! !

!XpfCtTagSearchHelper methodsFor: 'matching' stamp: 'cs 5/3/2009 17:16'!
matchAttributeBad

	self addTypeMatcher: [:att | att tag = #bad].! !

!XpfCtTagSearchHelper methodsFor: 'matching' stamp: 'cs 5/3/2009 17:16'!
matchAttributeTalk

	self addTypeMatcher: [:att | att tag = #talk].! !

!XpfCtTagSearchHelper methodsFor: 'matching' stamp: 'cs 5/4/2009 02:39'!
matchCreator: aString

	self addAttributeMatcher: [:att | att creator = aString].! !

!XpfCtTagSearchHelper methodsFor: 'matching' stamp: 'cs 5/4/2009 02:28'!
matchKeyword: aString

	self addAttributeMatcher: [:att | att includesChatLine: aString].! !

!XpfCtTagSearchHelper methodsFor: 'matching' stamp: 'cs 5/3/2009 14:07'!
matchTime: aSymbol

	self addAttributeMatcher: (self timeMatchers at: aSymbol).! !

!XpfCtTagSearchHelper methodsFor: 'accessing' stamp: 'cs 5/2/2009 15:45'!
methodMatchers

	^ methodMatchers! !

!XpfCtTagSearchHelper methodsFor: 'accessing' stamp: 'cs 5/2/2009 15:46'!
methodMatchers: anObject

	methodMatchers := anObject.! !

!XpfCtTagSearchHelper methodsFor: 'accessing' stamp: 'cs 5/2/2009 13:06'!
pvtResults

	^ results! !

!XpfCtTagSearchHelper methodsFor: 'accessing' stamp: 'cs 5/2/2009 13:07'!
results

	^ results asSortedCollection! !

!XpfCtTagSearchHelper methodsFor: 'accessing' stamp: 'cs 5/2/2009 12:39'!
results: anObject

	results := anObject.! !

!XpfCtTagSearchHelper methodsFor: 'searching' stamp: 'mt 5/5/2009 20:00'!
searchAttribute: anAttribute
	"Attributes may be a specific timestamp, creator or keyword."
	
	(self searchType: anAttribute) ifFalse: [^ false].
	self attributeMatchers do: [ :matcher |
		(matcher value: anAttribute) ifFalse: [^ false]].
	^ true! !

!XpfCtTagSearchHelper methodsFor: 'searching' stamp: 'mt 5/5/2009 21:31'!
searchCategory: aString

	| result |
	result := OrderedCollection new.
	Smalltalk organization categories
		select: [:c | c startsWith: aString]
		thenDo: [:c | result addAll: (Smalltalk organization listAtCategoryNamed: c)].
	self
		doWithProgress: result
		block: [:name | self searchClass: (Smalltalk at: name)]
		message: 'Searching category ', aString, '...'.! !

!XpfCtTagSearchHelper methodsFor: 'searching' stamp: 'mt 5/5/2009 21:31'!
searchClass: class

	class selectorsDo: [:sel |
		(self searchMethod: sel text: (class sourceCodeAt: sel))
			ifTrue: [self addToResultClass: class symbol: sel]].! !

!XpfCtTagSearchHelper methodsFor: 'searching' stamp: 'mt 5/5/2009 21:31'!
searchGlobal

	self
		doWithProgress: Smalltalk classNames
		block: [:className | self searchClass: (Smalltalk at: className)]
		message: 'Searching all source code...'.! !

!XpfCtTagSearchHelper methodsFor: 'searching' stamp: 'cs 5/2/2009 16:56'!
searchMethod: sel text: text
	
	text isText ifFalse: [^ false].
	self methodMatchers do: [:matcher |
		(matcher value: sel) ifFalse: [^ false]].
	^ self searchText: text! !

!XpfCtTagSearchHelper methodsFor: 'searching' stamp: 'mt 5/5/2009 19:59'!
searchText: aText

	aText runs values do: [:atts |
		atts do: [:att |
			((att isKindOf: XpfComplexAttribute)
			and: [self searchAttribute: att])
				ifTrue: [^ true]]].
	^ false! !

!XpfCtTagSearchHelper methodsFor: 'searching' stamp: 'cs 5/3/2009 17:17'!
searchType: anAttribute
		
	self typeMatchers do: [ :matcher |
		(matcher value: anAttribute) ifTrue: [^ true]].
	^ false! !

!XpfCtTagSearchHelper methodsFor: 'accessing' stamp: 'cs 5/4/2009 02:14'!
timeMatchers

	^ Dictionary
		with: #today -> [:att | att timestamp asDate = Date today]
		with: #yesterday -> [:att | att timestamp asDate = Date yesterday]
		with: #thisWeek -> [:att | att timestamp asWeek = Week current]
		with: #lastWeek -> [:att | att timestamp asWeek = (Week current - (Duration weeks: 1))]
		with: #thisMonth -> [:att | att timestamp asMonth = Month current]
		with: #lastMonth -> [:att | att timestamp asMonth = Date current onPreviousMonth asMonth]! !

!XpfCtTagSearchHelper methodsFor: 'accessing' stamp: 'cs 5/3/2009 17:15'!
typeMatchers

	^ typeMatchers! !

!XpfCtTagSearchHelper methodsFor: 'accessing' stamp: 'cs 5/3/2009 17:15'!
typeMatchers: anObject

	typeMatchers := anObject.! !

Object subclass: #XpfCtTagSearchModel
	instanceVariableNames: 'results category theClass findBad findTalk keyword creator selectedTime controller isSearching progressString progressValue progressValueMax searchProcess'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Models'!

!XpfCtTagSearchModel methodsFor: 'testing' stamp: 'mt 5/5/2009 21:33'!
canSearch

	^ self isSearching or: [(self findBad | self findTalk) &
		(self validClass and: [self validCategory])]! !

!XpfCtTagSearchModel methodsFor: 'accessing' stamp: 'cs 5/3/2009 13:26'!
category

	^ category asString asText
		addAttribute: (self validCategory
			ifTrue: [TextColor black]
			ifFalse: [TextColor red]);
		yourself! !

!XpfCtTagSearchModel methodsFor: 'accessing' stamp: 'mt 5/6/2009 10:07'!
category: aText
	
	category := aText asString withBlanksTrimmed asSymbol.
	self changed: #category.
	self changed: #canSearch.
	self changed: #theClassEnabled.! !

!XpfCtTagSearchModel methodsFor: 'accessing' stamp: 'cs 5/4/2009 10:45'!
categoryEnabled

	^ theClass isEmpty! !

!XpfCtTagSearchModel methodsFor: 'accessing' stamp: 'cs 5/3/2009 13:32'!
categoryLength

	^ {category size + 1}! !

!XpfCtTagSearchModel methodsFor: 'searching' stamp: 'cs 5/4/2009 00:45'!
children

	^ self results collect: [:ref |
		OBMethodNode on: ref]! !

!XpfCtTagSearchModel methodsFor: 'searching' stamp: 'mt 5/5/2009 21:45'!
clearResults

	results := OrderedCollection new.
	self changed: #results.! !

!XpfCtTagSearchModel methodsFor: 'ecompletion' stamp: 'cs 5/5/2009 17:30'!
completionController

	controller ifNil:[controller := ECController model: self ].
	^ controller! !

!XpfCtTagSearchModel methodsFor: 'accessing' stamp: 'cs 5/4/2009 09:48'!
creator
	^ creator! !

!XpfCtTagSearchModel methodsFor: 'accessing' stamp: 'cs 5/4/2009 09:48'!
creator: anObject
	creator := anObject! !

!XpfCtTagSearchModel methodsFor: 'searching' stamp: 'cs 5/4/2009 10:05'!
enableParams: helper
	
	self keyword ifNotEmpty: [helper matchKeyword: self keyword].
	self creator ifNotEmpty: [helper matchCreator: self creator].
	self selectedTime = #all ifFalse: [helper matchTime: self selectedTime].! !

!XpfCtTagSearchModel methodsFor: 'searching' stamp: 'mt 5/5/2009 15:11'!
enableTags: helper
	
	self findBad ifTrue: [helper matchAttributeBad ].
	self findTalk ifTrue: [helper matchAttributeTalk ].! !

!XpfCtTagSearchModel methodsFor: 'accessing' stamp: 'cs 5/2/2009 17:15'!
findBad

	^ findBad! !

!XpfCtTagSearchModel methodsFor: 'accessing' stamp: 'cs 5/2/2009 17:56'!
findBad: anObject

	findBad := anObject.
	self changed: #findBad.
	self changed: #canSearch.! !

!XpfCtTagSearchModel methodsFor: 'accessing' stamp: 'cs 5/2/2009 17:15'!
findTalk

	^ findTalk! !

!XpfCtTagSearchModel methodsFor: 'accessing' stamp: 'cs 5/2/2009 17:56'!
findTalk: anObject

	findTalk := anObject.
	self changed: #findTalk.
	self changed: #canSearch.! !

!XpfCtTagSearchModel methodsFor: 'initialization' stamp: 'mt 5/5/2009 15:13'!
initialize

	super initialize.
	self
		category: '';
		theClass: '';
		keyword: '';
		creator: '';
		findBad: true;
		findTalk: false;
		selectedTime: #all.! !

!XpfCtTagSearchModel methodsFor: 'accessing' stamp: 'mt 5/5/2009 20:29'!
isSearching

	isSearching ifNil: [isSearching := false].
	^ isSearching! !

!XpfCtTagSearchModel methodsFor: 'accessing' stamp: 'mt 5/5/2009 21:38'!
isSearching: aBoolean

	isSearching := aBoolean.
	self
		changed: #canSearch;
		changed: #searchButtonLabel.! !

!XpfCtTagSearchModel methodsFor: 'accessing' stamp: 'cs 5/2/2009 17:15'!
keyword

	^ keyword! !

!XpfCtTagSearchModel methodsFor: 'accessing' stamp: 'cs 5/2/2009 17:15'!
keyword: anObject

	keyword := anObject.! !

!XpfCtTagSearchModel methodsFor: 'accessing' stamp: 'cs 5/4/2009 09:51'!
possibleTimes
	
	^ #(all today yesterday thisWeek lastWeek thisMonth lastMonth).! !

!XpfCtTagSearchModel methodsFor: 'updating' stamp: 'mt 5/5/2009 21:02'!
progressString

	progressString ifNil: [progressString := ''].
	^ progressString! !

!XpfCtTagSearchModel methodsFor: 'updating' stamp: 'mt 5/5/2009 21:03'!
progressValue

	progressValue ifNil: [progressValue := 0].
	^ progressValue! !

!XpfCtTagSearchModel methodsFor: 'updating' stamp: 'mt 5/5/2009 21:04'!
progressValueMax

	progressValueMax ifNil: [progressValueMax := 100].
	^ progressValueMax! !

!XpfCtTagSearchModel methodsFor: 'searching' stamp: 'mt 5/5/2009 20:08'!
results

	results ifNil: [results := OrderedCollection new].
	^ results! !

!XpfCtTagSearchModel methodsFor: 'searching' stamp: 'mt 5/5/2009 21:48'!
search

	| helper |
	self clearResults.
	
	helper := XpfCtTagSearchHelper new.
	helper addDependent: self.
	
	self isSearching: true.
	self enableTags: helper.
	self enableParams: helper.
	searchProcess := [(self theClass notEmpty and: [self validClass])
		ifTrue: [helper searchClass: (Smalltalk at: theClass)]
		ifFalse: [(self category notEmpty and: [self validCategory])
			ifTrue: [helper searchCategory: self category]
			ifFalse: [helper searchGlobal]].
	self isSearching: false] fork priority: 35. "Lower than UI-Thread"! !

!XpfCtTagSearchModel methodsFor: 'accessing' stamp: 'mt 5/5/2009 21:37'!
searchButtonLabel

	^ self isSearching
		ifTrue: ['Stop Search']
		ifFalse: ['Start Search']! !

!XpfCtTagSearchModel methodsFor: 'searching' stamp: 'mt 5/5/2009 21:41'!
searchOrStop

	self isSearching
		ifTrue: [self searchStop]
		ifFalse: [self search].! !

!XpfCtTagSearchModel methodsFor: 'searching' stamp: 'mt 5/6/2009 10:25'!
searchStop

	searchProcess ifNil: [^ self].
	searchProcess terminate.
	searchProcess := nil.
	progressValue := 0.
	progressString := ''.
	self
		isSearching: false;
		changed: #progressValue;
		changed: #progressString.! !

!XpfCtTagSearchModel methodsFor: 'searching' stamp: 'cs 5/4/2009 00:51'!
selectResult: result

	(self children at: result) browse! !

!XpfCtTagSearchModel methodsFor: 'ecompletion' stamp: 'cs 5/5/2009 17:38'!
selectedClassOrMetaClass

	^ nil "I don't know any better"! !

!XpfCtTagSearchModel methodsFor: 'accessing' stamp: 'cs 5/4/2009 10:03'!
selectedTime

	^ selectedTime! !

!XpfCtTagSearchModel methodsFor: 'accessing' stamp: 'cs 5/4/2009 10:03'!
selectedTime: anObject

	selectedTime := anObject.! !

!XpfCtTagSearchModel methodsFor: 'accessing' stamp: 'cs 5/4/2009 10:01'!
selectedTimeIndex

	^ self possibleTimes indexOf: self selectedTime! !

!XpfCtTagSearchModel methodsFor: 'accessing' stamp: 'cs 5/4/2009 10:03'!
selectedTimeIndex: aNumber

	^ self selectedTime: (self possibleTimes at: aNumber)! !

!XpfCtTagSearchModel methodsFor: 'accessing' stamp: 'cs 5/3/2009 13:03'!
theClass

	^ theClass asString asText
		addAttribute: (self validClass
			ifTrue: [TextColor black]
			ifFalse: [TextColor red]);
		yourself! !

!XpfCtTagSearchModel methodsFor: 'accessing' stamp: 'mt 5/6/2009 10:07'!
theClass: aText
	
	theClass := aText asString withBlanksTrimmed asSymbol.
	self changed: #theClass.
	self changed: #canSearch.! !

!XpfCtTagSearchModel methodsFor: 'accessing' stamp: 'cs 5/3/2009 13:29'!
theClassEnabled

	^ category isEmpty! !

!XpfCtTagSearchModel methodsFor: 'accessing' stamp: 'cs 5/3/2009 13:19'!
theClassLength

	^ {theClass size + 1}! !

!XpfCtTagSearchModel methodsFor: 'updating' stamp: 'mt 5/5/2009 20:31'!
update: aSymbol

	super update: aSymbol.
	
	aSymbol = #searchFinished ifTrue: [
		self isSearching: false].! !

!XpfCtTagSearchModel methodsFor: 'updating' stamp: 'mt 5/5/2009 21:05'!
update: aSymbol with: anObject

	| label value max |
	super update: aSymbol with: anObject.
	
	aSymbol = #resultFound ifTrue: [
		self results add: anObject.
		self changed: #results].
	
	aSymbol = #progressChanged ifTrue: [
		label := anObject first.
		value := anObject second.
		max := anObject third.
		
		progressString := label.
		progressValue := value.
		progressValueMax := max.
		
		self
			changed: #progressString;
			changed: #progressValue].! !

!XpfCtTagSearchModel methodsFor: 'accessing' stamp: 'cs 5/3/2009 13:31'!
validCategory

	^ category isEmpty or:
		[Smalltalk organization categories anySatisfy: [:c | c startsWith: category]]! !

!XpfCtTagSearchModel methodsFor: 'accessing' stamp: 'cs 5/3/2009 13:30'!
validClass

	^ theClass isEmpty or: [Smalltalk includesKey: theClass]! !

BorderedMorph subclass: #XpfCtCodeChatMorph
	instanceVariableNames: 'model chatLog textEntry addMessageButton closeButton title searchEntry searchButton'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Morphs'!

!XpfCtCodeChatMorph class methodsFor: 'constants' stamp: 'mt 4/30/2009 08:23'!
defaultExtent

	^ 200@150! !

!XpfCtCodeChatMorph class methodsFor: 'instance creation' stamp: 'mt 5/2/2009 16:42'!
forAttribute: anXpfAttribute

	^ self new
		attribute: anXpfAttribute! !

!XpfCtCodeChatMorph methodsFor: 'accessing' stamp: 'mt 5/2/2009 17:56'!
addMessageButton

	addMessageButton ifNil: [addMessageButton := IconicButton new
			color: (Color white alpha: 0.5);
			target: self textEntry;
			borderWidth: 0;
			labelGraphic: MenuIcons xpfCtCommentAddIcon;
			actionSelector: #accept;
			useRoundedCorners;
			setBalloonText: 'Add message'].
	^ addMessageButton! !

!XpfCtCodeChatMorph methodsFor: 'accessing' stamp: 'mt 5/2/2009 17:45'!
attribute: anXpfAttribute

	self model ifNil: [^ self].
	self model attribute: anXpfAttribute.
	self title contents: ('{1} - {2}' format: {
			anXpfAttribute timestamp printRelativeTimeStamp.
			anXpfAttribute creator}).
	self color: (anXpfAttribute color copy lighter lighter alpha: 0.7).! !

!XpfCtCodeChatMorph methodsFor: 'accessing' stamp: 'mt 5/3/2009 10:58'!
chatLog

	chatLog ifNil: [chatLog := (UITheme current
		newMorphListIn: self
		for: self model
		list: #messages
		getSelected: #lastIndex
		setSelected: nil
		getEnabled: nil
		help: nil)
		hResizing: #rigid;
		vResizing: #rigid;
		cellInset: 0;
		layoutInset: 0;
		borderWidth: 0;
		fillStyle: Color transparent;
		extent: self class defaultExtent.
		chatLog listMorph cellInset: 0].
	^ chatLog! !

!XpfCtCodeChatMorph methodsFor: 'accessing' stamp: 'mt 5/2/2009 17:57'!
closeButton

	closeButton ifNil: [closeButton := IconicButton new
			color: (Color white alpha: 0.5);
			target: self;
			borderWidth: 0;
			labelGraphic: MenuIcons xpfCtCrossIcon;
			actionSelector: #delete;
			useRoundedCorners;
			setBalloonText: 'Close chat window'].
	^ closeButton! !

!XpfCtCodeChatMorph methodsFor: 'initialization' stamp: 'mt 5/3/2009 10:31'!
initialize

	super initialize.
	self
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap;
		listDirection: #topToBottom;
		borderWidth: 1;
		borderColor: Color black;
		changeTableLayout;
		fillStyle: (Color yellow alpha: 0.8);
		layoutInset: 5;
		cellInset: 5;
		useRoundedCorners;
		initializeUserInterface.! !

!XpfCtCodeChatMorph methodsFor: 'initialization' stamp: 'mt 5/3/2009 14:20'!
initializeUserInterface

	self
		addMorphBack: (UITheme current newRowIn: self for: {
			self title.
			self closeButton});
		addMorphBack: (UITheme current newRowIn: self for: {
			self searchEntry.
			self searchButton});
		addMorphBack: self chatLog;
		addMorphBack: (UITheme current newRowIn: self for: {
			self textEntry.
			self addMessageButton}).! !

!XpfCtCodeChatMorph methodsFor: 'accessing' stamp: 'mt 4/29/2009 22:10'!
model

	model ifNil: [model := XpfCtCodeChatModel new].
	^ model! !

!XpfCtCodeChatMorph methodsFor: 'accessing' stamp: 'mt 5/3/2009 14:18'!
searchButton

	searchButton ifNil: [searchButton := IconicButton new
			color: (Color white alpha: 0.5);
			target: self searchEntry;
			borderWidth: 0;
			labelGraphic: MenuIcons xpfCtMagnifierIcon;
			actionSelector: #accept;
			useRoundedCorners;
			setBalloonText: 'Highlight messages with keyword.'].
	^ searchButton! !

!XpfCtCodeChatMorph methodsFor: 'accessing' stamp: 'mt 5/3/2009 14:19'!
searchEntry

	searchEntry ifNil: [searchEntry := UITheme current
		newTextEntryIn: self
		for: self model
		get: nil
		set: #searchString:
		class: String
		getEnabled: nil
		help: 'Type a keyword to hightlight messages which contain that keyword.'.
		searchEntry
			useRoundedCorners;
			borderWidth: 0;
			fillStyle: (Color white alpha: 0.5);
			setText: 'Look for keyword...'].
	^ searchEntry! !

!XpfCtCodeChatMorph methodsFor: 'accessing' stamp: 'mt 5/3/2009 10:56'!
textEntry

	textEntry ifNil: [textEntry := UITheme current
		newTextEntryIn: self
		for: self model
		get: #newMessage
		set: #addMessage:
		class: String
		getEnabled: nil
		help: 'Type a message and hit the return key or push the button to the right.'.
		textEntry
			useRoundedCorners;
			borderWidth: 0;
			fillStyle: (Color white alpha: 0.5);
			setText: 'Type message here...'].
	^ textEntry! !

!XpfCtCodeChatMorph methodsFor: 'accessing' stamp: 'mt 5/3/2009 10:42'!
title

	title ifNil: [title := UITheme current
			newLabelIn: self label: 'Code Chat'.
		title hResizing: #spaceFill.
		title assureExtension
			balloonText: 'Creation time and creator'].
	^ title! !

BorderedMorph subclass: #XpfCtDiagramMorph
	instanceVariableNames: 'graphs dimensions grid'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Morphs'!

!XpfCtDiagramMorph methodsFor: 'add/remove' stamp: 'mt 5/9/2009 15:37'!
addGraph: aGraphMorph

	self addMorph: aGraphMorph.
	aGraphMorph
		position: self position;
		diagram: self.! !

!XpfCtDiagramMorph methodsFor: 'add/remove' stamp: 'mt 5/9/2009 16:48'!
addGrid: aPoint

	self removeGrid.
	
	(self minX to: self maxX by: aPoint x) do: [:t | |newLine|
		newLine := LineMorph
			from: (self scaledPoint: t @ self minY)
			to: (self scaledPoint: t @ self maxY)
			color: Color gray
			width: 1.
		self addMorph: newLine.
		self grid add: newLine].
	
	(self minY to: self maxY by: aPoint y) do: [:t | |newLine|
		newLine := LineMorph
			from: (self scaledPoint: self minX @ t)
			to: (self scaledPoint: self maxX @ t)
			color: Color gray
			width: 1.
		self addMorph: newLine.
		self grid add: newLine].
	
	self
		addGridAxis;
		addLabel: aPoint * (2@1).! !

!XpfCtDiagramMorph methodsFor: 'add/remove' stamp: 'mt 5/9/2009 16:27'!
addGridAxis

	| newLine |
	newLine := LineMorph
		from: (self scaledPoint: self minX @ 0)
		to: (self scaledPoint: self maxX @ 0)
		color: Color gray
		width: 2.

	self addMorph: newLine.
	self grid add: newLine.
	
	newLine := LineMorph
		from: (self scaledPoint: 0 @ self minY)
		to: (self scaledPoint: 0 @ self maxY)
		color: Color gray
		width: 2.

	self addMorph: newLine.
	self grid add: newLine.! !

!XpfCtDiagramMorph methodsFor: 'add/remove' stamp: 'mt 5/9/2009 16:39'!
addLabel: aPoint

	(self minX to: self maxX by: aPoint x) do: [:t | |label|		
		label := (StringMorph contents: t asString)
			position: (self scaledPoint: t@0).
		label position: (label position x - (label width // 2)) @ (label position y + 5).
		self addMorph: label].
	
	(self minY to: self maxY by: aPoint y) do: [:t | |label|
		label := (StringMorph contents: t asString)
			position: (self scaledPoint: 0@t).
		label position: (label position x - label width - 5) @ (label position y - (label height // 2)).
		self addMorph: label].! !

!XpfCtDiagramMorph methodsFor: 'accessing' stamp: 'mt 5/9/2009 11:37'!
dimensions

	dimensions ifNil: [self dimensions: (0@100 corner: 100@0)].
	^ dimensions! !

!XpfCtDiagramMorph methodsFor: 'accessing' stamp: 'mt 5/9/2009 11:37'!
dimensions: aRectangle

	dimensions := aRectangle.! !

!XpfCtDiagramMorph methodsFor: 'accessing' stamp: 'mt 5/9/2009 10:18'!
graphs

	graphs ifNil: [self graphs: Dictionary new].
	^ graphs! !

!XpfCtDiagramMorph methodsFor: 'accessing' stamp: 'mt 5/9/2009 10:18'!
graphs: anObject

	graphs anObject.! !

!XpfCtDiagramMorph methodsFor: 'accessing' stamp: 'mt 5/9/2009 16:01'!
grid

	grid ifNil: [self grid: OrderedCollection new].
	^ grid! !

!XpfCtDiagramMorph methodsFor: 'accessing' stamp: 'mt 5/9/2009 16:10'!
grid: anObject

	grid := OrderedCollection new.! !

!XpfCtDiagramMorph methodsFor: 'initialization' stamp: 'mt 5/9/2009 16:42'!
initialize

	super initialize.
	self
		extent: 200@200;
		color: Color white;
		borderWidth: 0.! !

!XpfCtDiagramMorph methodsFor: 'accessing' stamp: 'mt 5/9/2009 11:39'!
maxX

	^ self dimensions right! !

!XpfCtDiagramMorph methodsFor: 'accessing' stamp: 'mt 5/9/2009 11:39'!
maxY

	^ self dimensions top! !

!XpfCtDiagramMorph methodsFor: 'accessing' stamp: 'mt 5/9/2009 11:38'!
minX

	^ self dimensions left! !

!XpfCtDiagramMorph methodsFor: 'accessing' stamp: 'mt 5/9/2009 11:38'!
minY

	^ self dimensions bottom! !

!XpfCtDiagramMorph methodsFor: 'add/remove' stamp: 'mt 5/9/2009 16:11'!
removeGrid

	self grid copy do: [:gridLine | gridLine delete]! !

!XpfCtDiagramMorph methodsFor: 'scaling' stamp: 'mt 5/9/2009 16:03'!
scaledPoint: aPoint
	"Scale the point according to the diagrams dimensions."
	
	| scaledPoint |
	
	scaledPoint :=
		(((aPoint x - self minX) * self width)  // (self maxX abs + self minX abs)) @
		(((aPoint y - self minY) * self height) // (self maxY abs + self minY abs)).
		
	^ scaledPoint * (1@-1) + (0@self height) + self position! !

!Integer methodsFor: '*xpf-codetalk' stamp: 'mt 5/3/2009 21:24'!
asByteArrayOfSize2: aSize 
	"Answer a ByteArray of aSize with my value, most-significant byte first."
	| answer digitPos |
	aSize < self digitLength ifTrue: [ Error signal: 'number to large for byte array' ].
	answer := ByteArray new: aSize.
	digitPos := 1.
	aSize 
		to: aSize - self digitLength + 1
		by: -1
		do: 
			[ : pos | 
			answer 
				at: pos
				put: (self digitAt: digitPos).
			digitPos := digitPos + 1 ].
	^ answer! !

LabelMorph subclass: #XpfCtPluggableLabelMorph
	instanceVariableNames: 'labelSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Morphs'!

!XpfCtPluggableLabelMorph methodsFor: 'accessing' stamp: 'mt 5/6/2009 10:18'!
labelSelector
	^ labelSelector! !

!XpfCtPluggableLabelMorph methodsFor: 'accessing' stamp: 'mt 5/6/2009 10:22'!
labelSelector: anObject

	labelSelector := anObject.
	self update: anObject! !

!XpfCtPluggableLabelMorph methodsFor: 'updating' stamp: 'mt 5/6/2009 10:24'!
update: aSymbol

	super update: aSymbol.
	
	aSymbol = self labelSelector ifTrue: [self updateLabel].! !

!XpfCtPluggableLabelMorph methodsFor: 'updating' stamp: 'mt 5/6/2009 10:24'!
updateLabel

	| string |
	string := self model perform: self labelSelector.
	
	self 
		disableTableLayout: string isEmpty;
		visible: string isEmpty not.		
	
	self visible ifTrue:
		[self contents: string].! !

!OBMethodNode methodsFor: '*xpf-codetalk' stamp: 'mt 5/3/2009 10:02'!
categoryText

	^ self category! !

!OBMethodNode methodsFor: '*xpf-codetalk' stamp: 'mt 5/25/2009 14:34'!
remoteVersions

	| result lastDate |
	self metaNode filters ifEmpty: [^ #()].
	
	result := OrderedCollection new.
	
	self metaNode filters first cache do: [:ver | |t|
		t := ver definition fullTimeStamp.
		lastDate = t date ifFalse: [
			result add: (XpfCtSeparatorNode text: t date printRelativeDate)].
		lastDate := t date.
		result add: (OBMethodVersionNode on: ver)].

	^ result! !

!OBMethodNode methodsFor: '*xpf-codetalk' stamp: 'mt 5/3/2009 09:55'!
sourceText

	^ self source! !

PluggableListMorph subclass: #XpfPluggableListMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk'!

!XpfPluggableListMorph methodsFor: 'as yet unclassified' stamp: 'cs 5/4/2009 00:49'!
backgroundColorAt: row

	^ Color white! !

!XpfPluggableListMorph methodsFor: 'as yet unclassified' stamp: 'cs 5/4/2009 00:49'!
iconAt: row

	^ nil! !

!XpfPluggableListMorph methodsFor: 'as yet unclassified' stamp: 'cs 5/4/2009 00:37'!
listMorphClass

	^ XpfLazyListMorph! !

!OBPluggableListMorph methodsFor: '*xpf-codetalk-override' stamp: 'mt 4/26/2009 21:32'!
listMorphClass
    ^XpfLazyListMorph! !

PolygonMorph subclass: #XpfCtGraphMorph
	instanceVariableNames: 'model diagram'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Morphs'!

!XpfCtGraphMorph class methodsFor: 'instance creation' stamp: 'mt 5/9/2009 16:15'!
newFor: aGraphModel color: aColor

	^ (self new
		vertices: {0@0} asOrderedCollection
		color: Color black
		borderWidth: 3
		borderColor: aColor)
		makeOpen;
		model: aGraphModel! !

!XpfCtGraphMorph methodsFor: 'add/remove' stamp: 'mt 5/9/2009 16:04'!
addPoint: aPoint

	"(self diagram dimensions containsPoint: aPoint)
		ifFalse: [^ self]."

	self vertices add: (self diagram scaledPoint: aPoint).
	self computeBounds.! !

!XpfCtGraphMorph methodsFor: 'accessing' stamp: 'mt 5/9/2009 15:37'!
diagram

	^ diagram! !

!XpfCtGraphMorph methodsFor: 'accessing' stamp: 'mt 5/9/2009 15:37'!
diagram: aDiagramMorph

	diagram := aDiagramMorph.! !

!XpfCtGraphMorph methodsFor: 'accessing' stamp: 'mt 5/9/2009 11:36'!
model

	^ model! !

!XpfCtGraphMorph methodsFor: 'accessing' stamp: 'mt 5/9/2009 14:52'!
model: anObject

	self model ifNotNilDo: [:m | m removeDependent: self].
	model := anObject.
	self model ifNotNilDo: [:m | m addDependent: self].! !

!XpfCtGraphMorph methodsFor: 'add/remove' stamp: 'mt 5/9/2009 16:04'!
removeAllPoints

	vertices := {self diagram scaledPoint: 0@0} asOrderedCollection.
	self computeBounds.! !

!XpfCtGraphMorph methodsFor: 'updating' stamp: 'mt 5/9/2009 14:55'!
update: aSymbol

	super update: aSymbol.
	aSymbol = #reset ifTrue: [self removeAllPoints].! !

!XpfCtGraphMorph methodsFor: 'updating' stamp: 'mt 5/9/2009 15:03'!
update: aSymbol with: anObject

	super update: aSymbol with: anObject.
	aSymbol = #newPoint ifTrue: [self addPoint: anObject].! !

!OBNode methodsFor: '*xpf-codetalk' stamp: 'mt 5/25/2009 14:46'!
isSeparatorNode

   ^false! !

OBNode subclass: #XpfCtSeparatorNode
	instanceVariableNames: 'text'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Versions'!

!XpfCtSeparatorNode class methodsFor: 'instance creation' stamp: 'mt 5/25/2009 14:15'!
text: aString

	^ self new
		text: aString! !

!XpfCtSeparatorNode methodsFor: 'public' stamp: 'mt 5/25/2009 14:30'!
hasSelector

	^ false! !

!XpfCtSeparatorNode methodsFor: 'public' stamp: 'mt 5/25/2009 14:30'!
hasVersions

	^ false! !

!XpfCtSeparatorNode methodsFor: 'testing' stamp: 'mt 5/25/2009 14:47'!
isSeparatorNode

	^ true! !

!XpfCtSeparatorNode methodsFor: 'public' stamp: 'mt 5/25/2009 14:48'!
name

	^ text asText allBold! !

!XpfCtSeparatorNode methodsFor: 'accessing' stamp: 'mt 5/25/2009 14:15'!
text

	^ text! !

!XpfCtSeparatorNode methodsFor: 'accessing' stamp: 'mt 5/25/2009 14:14'!
text: anObject

	text := anObject.! !

ImageMorph subclass: #XpfCtAttributeIconMorph
	instanceVariableNames: 'attribute lastVisible'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Core'!

!XpfCtAttributeIconMorph methodsFor: 'accessing' stamp: 'mt 5/1/2009 17:19'!
attribute
	^ attribute! !

!XpfCtAttributeIconMorph methodsFor: 'accessing' stamp: 'mt 5/1/2009 17:19'!
attribute: anObject
	attribute := anObject! !

!XpfCtAttributeIconMorph methodsFor: 'event handling' stamp: 'mt 5/1/2009 17:19'!
handlesMouseDown: evt

	^ true! !

!XpfCtAttributeIconMorph methodsFor: 'accessing' stamp: 'mt 5/3/2009 19:43'!
visible: aBoolean

	lastVisible = aBoolean ifTrue: [^ self].
	lastVisible := aBoolean.
	self disableTableLayout: aBoolean not.
	^ super visible: aBoolean! !

XpfCtAttributeIconMorph subclass: #XpfCtChatIconMorph
	instanceVariableNames: 'chat'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Core'!

!XpfCtChatIconMorph methodsFor: 'accessing' stamp: 'mt 5/2/2009 15:16'!
balloonText

	^ 'View chat'! !

!XpfCtChatIconMorph methodsFor: 'accessing' stamp: 'mt 4/30/2009 14:49'!
chat

	chat ifNil: [chat := XpfCtCodeChatMorph forAttribute: self attribute].
	^ chat! !

!XpfCtChatIconMorph methodsFor: 'initialization' stamp: 'mt 5/1/2009 17:55'!
initialize

	super initialize.
	self image: (MenuIcons xpfCtCommentsIcon).! !

!XpfCtChatIconMorph methodsFor: 'event handling' stamp: 'mt 5/2/2009 17:54'!
mouseDown: evt

	self attribute ifNil: [^ self].
	self chat
		initialize;
		position: self positionInWorld;
		attribute: self attribute;
		openInWorld.! !

XpfCtAttributeIconMorph subclass: #XpfCtDeleteIconMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Core'!

!XpfCtDeleteIconMorph methodsFor: 'accessing' stamp: 'mt 5/2/2009 15:16'!
balloonText

	^ 'Delete markup'! !

!XpfCtDeleteIconMorph methodsFor: 'initialization' stamp: 'mt 5/1/2009 17:55'!
initialize

	super initialize.
	self image: (MenuIcons xpfCtDeleteIcon).! !

!XpfCtDeleteIconMorph methodsFor: 'event handling' stamp: 'mt 5/1/2009 17:55'!
mouseDown: evt

	self attribute ifNil: [^ self].
	self attribute requestDeletion.! !

XpfCtAttributeIconMorph subclass: #XpfCtResizeIconMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Core'!

!XpfCtResizeIconMorph methodsFor: 'accessing' stamp: 'mt 5/3/2009 19:17'!
balloonText

	^ 'Resize markup'! !

!XpfCtResizeIconMorph methodsFor: 'initialization' stamp: 'mt 5/3/2009 19:18'!
initialize

	super initialize.
	self image: (MenuIcons xpfCtPaintcanIcon).! !

!XpfCtResizeIconMorph methodsFor: 'event handling' stamp: 'mt 5/3/2009 19:19'!
mouseDown: evt

	self attribute ifNil: [^ self].
	self attribute requestResizing.! !

!OBPluggableTextMorphWithShout class methodsFor: '*xpf-codetalk-override' stamp: 'mt 5/1/2009 15:45'!
on: anObject text: getTextSel accept: setTextSel readSelection: getSelectionSel menu: getMenuSel
	"CodeTalk: Use the XpfStyler right now."
	
	|styler answer stylerClass |
	answer := self new.
	stylerClass := answer stylerClass.
	styler := stylerClass
		ifNotNil:[
			stylerClass new
				view: answer;
				yourself].
	^ answer
		styler: styler;
		on: anObject
		text: getTextSel
		accept: setTextSel
		readSelection: getSelectionSel
		menu: getMenuSel! !

!OBPluggableTextMorphWithShout methodsFor: '*xpf-codetalk' stamp: 'topa 2/13/2009 13:48'!
stylerClass
	^XpfTextStyler! !

!OBPluggableTextMorphWithShout methodsFor: '*xpf-codetalk-override' stamp: 'topa 2/13/2009 13:48'!
textMorphClass
	"Answer the class used to create the receiver's textMorph"
	^XpfTextMorph! !

!RunArray class methodsFor: '*xpf-codetalk-override' stamp: 'mt 4/30/2009 11:37'!
scanFrom: strm
	"Read the style section of a fileOut or sources file.  nextChunk has already been done.  We need to return a RunArray of TextAttributes of various kinds.  These are written by the implementors of writeScanOn:"
	| rr vv aa this |
	(strm peekFor: $( ) ifFalse: [^ nil].
	rr := OrderedCollection new.
	[strm skipSeparators.
	 strm peekFor: $)] whileFalse: 
		[rr add: (Number readFrom: strm)].
	vv := OrderedCollection new.	"Value array"
	aa := OrderedCollection new.	"Attributes list"
	[(this := strm next) == nil] whileFalse: [
		this == $, ifTrue: [vv add: aa asArray.  aa := OrderedCollection new].
		this == $a ifTrue: [aa add: 
			(TextAlignment new alignment: (Integer readFrom: strm))].
		this == $f ifTrue: [aa add: 
			(TextFontChange new fontNumber: (Integer readFrom: strm))].
		this == $F ifTrue: [aa add: (TextFontReference toFont: 
			(StrikeFont familyName: (strm upTo: $#) size: (Integer readFrom: strm)))].
		this == $b ifTrue: [aa add: (TextEmphasis bold)].
		this == $i ifTrue: [aa add: (TextEmphasis italic)].
		this == $u ifTrue: [aa add: (TextEmphasis underlined)].
		this == $= ifTrue: [aa add: (TextEmphasis struckOut)].
		this == $n ifTrue: [aa add: (TextEmphasis normal)].
		this == $- ifTrue: [aa add: (TextKern kern: -1)].
		this == $+ ifTrue: [aa add: (TextKern kern: 1)].
		this == $c ifTrue: [aa add: (TextColor scanFrom: strm)]. "color"
		this == $h ifTrue: [aa add: (XpfComplexAttribute scanFrom: strm)]. "background color - CodeTalk"
		this == $L ifTrue: [aa add: (TextLink scanFrom: strm)].	"L not look like 1"
		this == $R ifTrue: [aa add: (TextURL scanFrom: strm)].
				"R capitalized so it can follow a number"
		this == $q ifTrue: [aa add: (TextSqkPageLink scanFrom: strm)].
		this == $p ifTrue: [aa add: (TextSqkProjectLink scanFrom: strm)].
		this == $P ifTrue: [aa add: (TextPrintIt scanFrom: strm)].
		this == $d ifTrue: [aa add: (TextDoIt scanFrom: strm)].
		"space, cr do nothing"
		].
	aa size > 0 ifTrue: [vv add: aa asArray].
	^ self runs: rr asArray values: vv asArray
"
RunArray scanFrom: (ReadStream on: '(14 50 312)f1,f1b,f1LInteger +;i')
"! !

!MCMethodDefinition class methodsFor: '*xpf-codetalk-override' stamp: 'mt 4/20/2009 15:54'!
forMethodReference: aMethodReference
	"CodeTalk: Get the markup from the method reference and put it into the definition."

	| definition |
	definition := self cachedDefinitions at: aMethodReference compiledMethod ifAbsent: [].
	(definition isNil
		or: [definition selector ~= aMethodReference methodSymbol]
		or: [definition className ~= aMethodReference classSymbol]
		or: [definition classIsMeta ~= aMethodReference classIsMeta]
		or: [definition category ~= aMethodReference category])
			ifTrue: [definition := (self 
						className: aMethodReference classSymbol
						classIsMeta: aMethodReference classIsMeta
						selector: aMethodReference methodSymbol
						category: aMethodReference category
						timeStamp: aMethodReference timeStamp
						source: aMethodReference source)
						sourceText: aMethodReference sourceCode.
					self cachedDefinitions at: aMethodReference compiledMethod put: definition].
	^ definition
	! !

!MCMethodDefinition methodsFor: '*xpf-codetalk-override' stamp: 'cs 4/17/2009 18:23'!
addMethodAdditionTo: aCollection
	| methodAddition |
	methodAddition := MethodAddition new
		compile: self sourceText
		classified: category
		withStamp: timeStamp
		notifying: (SyntaxError new category: category)
		logSource: true
		inClass: self actualClass.
	"This might raise an exception and never return"
	methodAddition createCompiledMethod.
	aCollection add: methodAddition.
! !

!MCMethodDefinition methodsFor: '*xpf-codetalk-override' stamp: 'cs 4/17/2009 18:24'!
load
	self actualClass
		compile: self sourceText
		classified: category
		withStamp: timeStamp
		notifying: (SyntaxError new category: category)! !

!MCMethodDefinition methodsFor: '*xpf-codetalk' stamp: 'cs 4/20/2009 16:16'!
sourceText

	^ sourceText ifNil: [self source asText]! !

!MCMethodDefinition methodsFor: '*xpf-codetalk' stamp: 'mt 3/24/2009 15:52'!
sourceText: aText

	sourceText := aText.! !

MultiNewParagraph subclass: #XpfParagraph
	instanceVariableNames: 'textMorph showLineNumbers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Core'!

!XpfParagraph methodsFor: 'displaying' stamp: 'mt 5/3/2009 19:36'!
addAttributeMorphs
	"Add the small iconmorphs that appear for some attributes in the text."
	
	| morphs |
	selectionStart isNil | self textMorph isNil
		ifTrue: [^ self].

	morphs := (self text
		attributesOfKind: XpfComplexAttribute 
		from: selectionStart stringIndex
		to: ((selectionStop stringIndex - 1) max: selectionStart stringIndex))
		collect: [:attr | self textMorph addMorph: attr morph. attr morph].
	self textMorph removeAllMorphsBut: morphs.! !

!XpfParagraph methodsFor: 'editing' stamp: 'mt 5/18/2009 11:12'!
changeTagInSelectionTo: aSymbol
	"Used to change the tag (#bad, #talk) )of an attribute in the text."

	| attrs |
	attrs := self text
		attributesOfKind: XpfComplexAttribute
		from: selectionStart stringIndex
		to: selectionStop stringIndex - 1.
		
	attrs ifEmpty: [^ false].
	attrs
		do: [:attr | attr tag: aSymbol].

	^ true! !

!XpfParagraph methodsFor: 'displaying' stamp: 'mt 5/3/2009 19:23'!
displayOn: aCanvas using: displayScanner at: somePosition

	self
		updateProperties;
		addAttributeMorphs;
		updateAttributeMorphs;
		drawLineNumberBorderOn: aCanvas.
	
	(self lineIndexForPoint: aCanvas clipRect topLeft)
		to: (self lineIndexForPoint: aCanvas clipRect bottomRight)
		do: [ :i | | line |
			line := lines at: i.
			self drawComplexAttributeOn: aCanvas at: i.
			self drawLineNumber: i on: aCanvas for: line].
	
	super displayOn: aCanvas using: displayScanner at: somePosition.! !

!XpfParagraph methodsFor: 'displaying' stamp: 'mt 5/3/2009 11:12'!
drawComplexAttributeOn: aCanvas at: index 

	| textLine line nextLine|
	line := lines at: index.
	nextLine := lines size > index ifTrue: [lines at: index + 1] ifFalse: [].
	textLine := self text 
		copyFrom: line first
		to: line last.
	textLine runs withStartStopAndValueDo: [:start :stop :attributes |
		(attributes 
			detect: [:attr | attr isKindOf: XpfComplexAttribute]
			ifNone: [])
			ifNotNilDo: [:cAttr | | cb1 cb2 |
				cb1 := self characterBlockForIndex: start + (line first - 1).
				cb2 := self characterBlockForIndex: stop + (line first - 1).
	
				"Highlight the attribute."
				cAttr
					displayOn: aCanvas
					in: (cb1 left @ line top corner: cb2 right @ line bottom).

				"Update position of the morph from the attribute."
				(self isLastOccuranceOf: cAttr in: textLine position: stop for: index)
					ifTrue: [self
						moveAttributeMorph: cAttr morph
						to: cb2 right @ (line top - cAttr morph height)]]].! !

!XpfParagraph methodsFor: 'displaying' stamp: 'mt 5/1/2009 16:15'!
drawLineNumber: aNumber on: aCanvas for: line

	| str |
	showLineNumbers ifFalse: [^ self].
	self textMorph ifNil: [^ self].
	
	str := (aNumber < 10 ifTrue: [' '] ifFalse: ['']), aNumber asString.
	aCanvas
		drawString: str
		from: 1
		to: str size
		at: (self textMorph position x @ (line top + 2))
		font: Preferences standardCodeFont
		color: ((self isInSelectionFor: line)
			ifTrue: [Color black]
			ifFalse: [Color gray]).! !

!XpfParagraph methodsFor: 'displaying' stamp: 'mt 5/1/2009 16:12'!
drawLineNumberBorderOn: aCanvas

	| fontWidth |
	showLineNumbers ifFalse: [^ self].
	self textMorph ifNil: [^ self].
	
	fontWidth := Preferences standardCodeFont widthOfString: '99'.
	aCanvas
		line: (self textMorph position x + fontWidth + 3) @ aCanvas clipRect top
		to: (self textMorph position x + fontWidth + 3) @ aCanvas clipRect bottom
		width: 1
		color: Color gray.! !

!XpfParagraph methodsFor: 'testing' stamp: 'mt 5/1/2009 13:39'!
isInSelectionFor: line

	(selectionStart isNil or: [selectionStop isNil])
		ifTrue: [^ false].
		
	^ line last >= selectionStart textLine first
		and: [line first <= selectionStop textLine last]! !

!XpfParagraph methodsFor: 'testing' stamp: 'mt 5/1/2009 15:31'!
isLastOccuranceOf: cAttr in: textLine position: endOfRunIndex for: lineIndex
	"Answer true if the attribute does not appear in another run past the current one."

	| nextLine |
	nextLine := lines size > lineIndex ifTrue: [lines at: lineIndex + 1] ifFalse: [].
	^ ((nextLine isNil
		or: [((self text attributesAt: nextLine first) noneSatisfy: [:att | att == cAttr])])
		and: [textLine runs size < (endOfRunIndex + 1) 
			or: [(textLine attributesAt: endOfRunIndex + 1)	noneSatisfy: [:att | att == cAttr]]])! !

!XpfParagraph methodsFor: 'displaying' stamp: 'mt 5/3/2009 13:53'!
moveAttributeMorph: aMorph to: aPosition
	"Used to display small morphs to interaction with the markup. Pay attention to the extent of the textMorph if possible."
	
	| pos |
	pos := aPosition.
	self textMorph ifNotNilDo: [:t |
		pos := (t extent x - aMorph width min: aPosition x)
			@ (aPosition y < -5 ifTrue: [aMorph height max: aPosition y] ifFalse: [aPosition y])].
	aMorph position: pos.! !

!XpfParagraph methodsFor: 'editing' stamp: 'mt 5/3/2009 19:04'!
resizeTagInSelection
	"Used to grow or shrink the extent of an attribute in the text."

	| attrs |
	selectionStart = selectionStop ifTrue: [^ false].
	attrs := self text
		attributesOfKind: XpfComplexAttribute
		from: selectionStart stringIndex
		to: selectionStop stringIndex - 1.
	
	attrs size > 1
		ifTrue: [^ false. "Can only modify one attribute here."].
		
	self text
		removeAttribute: attrs first;
		addAttribute: attrs first
		from: selectionStart stringIndex
		to: selectionStop stringIndex - 1.
	^ true! !

!XpfParagraph methodsFor: 'editing' stamp: 'mt 5/3/2009 18:51'!
tagSelectionWith: aSymbol
	"Tag the selected code or text with the symbol."

	| attribute |
	selectionStart = selectionStop ifTrue: [^ false].
	attribute := XpfComplexAttribute forTag: aSymbol.
	attribute addDependent: self textMorph.
	self text
		addAttribute: attribute
		from: selectionStart stringIndex
		to: selectionStop stringIndex - 1.
	^ true! !

!XpfParagraph methodsFor: 'accessing' stamp: 'mt 4/22/2009 20:37'!
textMorph
	^ textMorph! !

!XpfParagraph methodsFor: 'accessing' stamp: 'mt 4/22/2009 20:37'!
textMorph: anObject
	textMorph := anObject! !

!XpfParagraph methodsFor: 'editing' stamp: 'mt 5/5/2009 15:02'!
untagSelection

	(self text
		attributesOfKind: XpfComplexAttribute
		from: selectionStart stringIndex
		to: selectionStop stringIndex) do: [:attr |
			self text
				removeAttribute: attr
				around: selectionStart stringIndex
				and: selectionStop stringIndex - 1].
	^ true! !

!XpfParagraph methodsFor: 'displaying' stamp: 'mt 5/18/2009 11:20'!
updateAttributeMorphs
	
	| attrs |
	selectionStart isNil | self textMorph isNil
		ifTrue: [^ self].
	
	attrs := self text
		attributesOfKind: XpfComplexAttribute 
		from: selectionStart stringIndex
		to: ((selectionStop stringIndex - 1) max: selectionStart stringIndex).

	(attrs size = 1) & (selectionStart ~= selectionStop)
		ifTrue: [attrs first resizeMorph visible: true]
		ifFalse: [attrs do: [:attr | attr resizeMorph visible: false]].! !

!XpfParagraph methodsFor: 'accessing' stamp: 'mt 5/1/2009 16:16'!
updateProperties

	showLineNumbers ifNil: [showLineNumbers := 
		(Preferences
			preferenceAt: #showLineNumbers
			ifAbsent: [Preference new rawValue: false])
			preferenceValue].! !

OBCommand subclass: #XpfCtCmdBrowseRemoteMethodVersions
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Commands'!

!XpfCtCmdBrowseRemoteMethodVersions methodsFor: 'as yet unclassified' stamp: 'mt 5/19/2009 15:27'!
execute

	XpfCtRemoteVersionBrowser openOn: target copy.! !

!XpfCtCmdBrowseRemoteMethodVersions methodsFor: 'as yet unclassified' stamp: 'mt 5/19/2009 15:26'!
group

	^ #navigation! !

!XpfCtCmdBrowseRemoteMethodVersions methodsFor: 'as yet unclassified' stamp: 'mt 5/19/2009 15:22'!
isActive

	^ target hasVersions and: [requestor isSelected: target]! !

!XpfCtCmdBrowseRemoteMethodVersions methodsFor: 'as yet unclassified' stamp: 'mt 5/18/2009 13:18'!
label

	^ 'remote versions'! !

!XpfCtCmdBrowseRemoteMethodVersions methodsFor: 'as yet unclassified' stamp: 'mt 5/18/2009 13:16'!
wantsButton

	^ true! !

OBCommand subclass: #XpfCtCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Commands'!

XpfCtCommand subclass: #XpfCtCmdChangeTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Commands'!

!XpfCtCmdChangeTag methodsFor: 'as yet unclassified' stamp: 'mt 5/18/2009 10:54'!
group

	^ '2'! !

!XpfCtCmdChangeTag methodsFor: 'as yet unclassified' stamp: 'mt 5/18/2009 11:05'!
isEnabled

	^ target fullText
		includesAttributesOfKind: XpfComplexAttribute
		from: target start
		to: (target stop max: target start)! !

!XpfCtCmdChangeTag methodsFor: 'as yet unclassified' stamp: 'mt 5/18/2009 10:55'!
label

	^ 'change tag'! !

!XpfCtCmdChangeTag methodsFor: 'as yet unclassified' stamp: 'mt 5/18/2009 10:55'!
mark: aTarget in: anEditor

	^ anEditor changeMarkupTag.! !

XpfCtCommand subclass: #XpfCtCmdJustTalk
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Commands'!

!XpfCtCmdJustTalk methodsFor: 'accessing' stamp: 'mt 5/2/2009 19:20'!
group

	^ '1'! !

!XpfCtCmdJustTalk methodsFor: 'accessing' stamp: 'mt 5/5/2009 15:01'!
icon

	^ #xpfCtMarkCommentIcon! !

!XpfCtCmdJustTalk methodsFor: 'accessing' stamp: 'mt 5/5/2009 15:38'!
keystroke

	^ $t! !

!XpfCtCmdJustTalk methodsFor: 'accessing' stamp: 'mt 5/5/2009 21:47'!
label

	^ 'add code chat'! !

!XpfCtCmdJustTalk methodsFor: 'execution' stamp: 'mt 5/2/2009 09:37'!
mark: aTarget in: anEditor

	^ anEditor justTalk: aTarget! !

XpfCtCommand subclass: #XpfCtCmdMarkBad
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Commands'!

!XpfCtCmdMarkBad methodsFor: 'accessing' stamp: 'mt 5/5/2009 15:00'!
group

	^ '1'! !

!XpfCtCmdMarkBad methodsFor: 'accessing' stamp: 'mt 5/5/2009 15:01'!
icon

	^ #xpfCtMarkBadIcon! !

!XpfCtCmdMarkBad methodsFor: 'accessing' stamp: 'mt 5/5/2009 15:38'!
keystroke

	^ $r! !

!XpfCtCmdMarkBad methodsFor: 'accessing' stamp: 'mfp 2/3/2009 16:11'!
label
	^ 'mark bad'! !

!XpfCtCmdMarkBad methodsFor: 'editor-bridge' stamp: 'mt 5/1/2009 16:00'!
mark: aTarget in: anEditor

	^ anEditor markAsBad: aTarget.! !

XpfCtCommand subclass: #XpfCtCmdRemoveMarkup
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Commands'!

!XpfCtCmdRemoveMarkup methodsFor: 'accessing' stamp: 'mt 5/5/2009 15:00'!
group

	^ '2'! !

!XpfCtCmdRemoveMarkup methodsFor: 'accessing' stamp: 'mt 5/2/2009 19:23'!
icon

	^ #xpfCtDeleteIcon! !

!XpfCtCmdRemoveMarkup methodsFor: 'testing' stamp: 'mt 5/3/2009 18:44'!
isEnabled

	^ target fullText
		includesAttributesOfKind: XpfComplexAttribute
		from: target start
		to: (target stop max: target start)! !

!XpfCtCmdRemoveMarkup methodsFor: 'accessing' stamp: 'mt 5/3/2009 19:06'!
label

	^'remove markup'! !

!XpfCtCmdRemoveMarkup methodsFor: 'execution' stamp: 'mt 5/1/2009 16:00'!
mark: aTarget in: anEditor
	
	anEditor removeMarkup: aTarget.
	^ true! !

XpfCtCommand subclass: #XpfCtCmdResize
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-CodeTalk-Commands'!

!XpfCtCmdResize methodsFor: 'accessing' stamp: 'mt 5/5/2009 15:00'!
group

	^ '2'! !

!XpfCtCmdResize methodsFor: 'testing' stamp: 'mt 5/3/2009 19:17'!
icon

	^ #xpfCtPaintcanIcon! !

!XpfCtCmdResize methodsFor: 'testing' stamp: 'mt 5/18/2009 11:20'!
isEnabled

	^ target start <= target stop
		and: [(target fullText
			attributesOfKind: XpfComplexAttribute
			from: target start
			to: target stop) size = 1]! !

!XpfCtCmdResize methodsFor: 'accessing' stamp: 'mt 5/3/2009 19:06'!
label

	^'resize markup'! !

!XpfCtCmdResize methodsFor: 'execution' stamp: 'mt 5/3/2009 19:07'!
mark: aTarget in: anEditor
	
	anEditor resizeMarkup: aTarget.
	^ true! !

!XpfCtCommand class methodsFor: 'testing' stamp: 'cs 5/2/2009 13:30'!
takesText

	^ true! !

!XpfCtCommand methodsFor: 'accessing' stamp: 'cs 5/2/2009 13:31'!
cluster

	^ #CodeTalk! !

!XpfCtCommand methodsFor: 'execution' stamp: 'cs 5/2/2009 13:32'!
execute
	"'inspired' by ORCmdPrettyPrint>>execute"

	| panel morph editor |	
	panel := requestor browser panels 
		detect: [ :each | each isKindOf: OBDefinitionPanel ]
		ifNone: [ ^ self ].
	morph := OBPluggableTextMorph allSubInstances
		detect: [ :each | each model == panel ]
		ifNone: [ ^ self ].
	editor := morph textMorph editor.
	morph hasUnacceptedEdits: (self mark: target in: editor).! !

!XpfCtCommand methodsFor: 'testing' stamp: 'mt 5/2/2009 09:59'!
isActive

	^ target isKindOf: OBTextSelection! !

!XpfCtCommand methodsFor: 'testing' stamp: 'mt 5/3/2009 18:53'!
isEnabled

	^ target stop >= target start
		and: [(target fullText
			includesAttributesOfKind: XpfComplexAttribute
			from: target start
			to: target stop) not]! !

!XpfCtCommand methodsFor: 'execution' stamp: 'cs 5/2/2009 13:32'!
mark: aTarget in: anEditor

	^ self subclassResponsibility! !
XpfCodeTalk initialize!
